@@ -22,7 +22,6 @@
 #include "ttconfig.h"
 
 #include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
 
 #ifdef HAVE_UNISTD_H
@@ -51,13 +50,14 @@
 
 #include "freetype.h"
 #include "tttypes.h"
-#include "tterror.h"
+#include "ttdebug.h"
 #include "ttengine.h"
 #include "ttmutex.h"
 #include "ttmemory.h"
 #include "ttfile.h"     /* our prototypes */
 
   /* This definition is mandatory for each file component! */
+  EXPORT_FUNC
   const TFileFrame  TT_Null_FileFrame = { NULL, 0, 0 };
   
   /* It has proven useful to do some bounds checks during   */
@@ -65,28 +65,28 @@
   /* this component to enable them.                         */
 
 #ifdef DEBUG_FILE
-#define CHECK_FRAME( frame, n )                                \
-          do {                                                 \
-            if ( frame.cursor+n > frame.address + frame.size ) \
-              Panic( "Frame boundary error!\n" );              \
-          } while ( 0 )
+#define CHECK_FRAME( frame, n )                        \
+  do {                                                 \
+    if ( frame.cursor+n > frame.address + frame.size ) \
+      Panic( "Frame boundary error!\n" );              \
+  } while ( 0 )
 #else
 #define CHECK_FRAME( frame, n ) \
-              do {              \
-              } while( 0 )
+  do {                          \
+  } while( 0 )
 #endif
 
   struct _TFileMap
   {
-    char*   base;       /* base address of mapped file       */
-    int     refcount;   /* reference count for mmaped region */
-    long    size;       /* stream size in file               */
-    long    offset;     /* offset in file                    */
+    String*  base;       /* base address of mapped file       */
+    Int      refcount;   /* reference count for mmaped region */
+    Long     size;       /* stream size in file               */
+    Long     offset;     /* offset in file                    */
   };
 
   typedef struct _TFileMap  TFileMap;
 
-#define MAP_Address( map )  (unsigned char*)( (map)->base + (map)->offset )
+#define MAP_Address( map )  (Byte*)( (map)->base + (map)->offset )
 
   /* The stream record structure */
   typedef struct _TStream_Rec
@@ -100,7 +100,7 @@
 #define STREAM2REC( x )  ( (TStream_Rec*)HANDLE_Val( x ) )
 
 
-#ifndef TT_CONFIG_REENTRANT
+#ifndef TT_CONFIG_OPTION_THREAD_SAFE
   
   /*******************************************************************/
   /*******************************************************************/
@@ -130,14 +130,14 @@
 
 /* The macro CUR_Stream denotes the current input stream              */
 /* Note that for the re-entrant version, the 'stream' name has been   */
-/* chosen according to the macro STREAM_ARGS                          */
+/* chosen according to the macro STREAM_ARGS.                         */
 
 /* The macro CUR_Frame denotes the current file frame               */
 /* Note that for the re-entrant version, the 'frame' name has been  */
-/* chosen according to the macro FRAME_ARGS                         */
+/* chosen according to the macro FRAME_ARGS.                        */
 
 /* The macro STREAM_VAR is used when calling public functions */
-/* that need an 'optional' stream argument                    */
+/* that need an 'optional' stream argument.                   */
 
 #define CUR_Stream   files.stream            /* thread-safe macros */
 #define CUR_Frame    files.frame
@@ -146,6 +146,7 @@
 #define STREAM_VAR   /* void */
   
   /* the 'files' variable is only defined in non-reentrant builds */
+
   static TFile_Component  files;
 
 
@@ -158,7 +159,8 @@
  *
  ******************************************************************/
 
- TT_Error TTFile_Init( PEngine_Instance  engine )
+ LOCAL_FUNC
+ TT_Error  TTFile_Init( PEngine_Instance  engine )
  {
    MUTEX_Create( files.lock );
    files.stream = NULL;
@@ -176,6 +178,7 @@
  *
  ******************************************************************/
 
+ LOCAL_FUNC
  TT_Error  TTFile_Done( PEngine_Instance  engine )
  {
    MUTEX_Destroy( files.lock );
@@ -197,6 +200,7 @@
  *
  ******************************************************************/
 
+  EXPORT_FUNC
   TT_Error  TT_Use_Stream( TT_Stream   org_stream,
                            TT_Stream*  stream )
   {
@@ -220,6 +224,7 @@
  *
  ******************************************************************/
 
+ EXPORT_FUNC
  TT_Error  TT_Done_Stream( TT_Stream*  stream )
  {
    HANDLE_Set( *stream, NULL );
@@ -228,7 +233,7 @@
    return TT_Err_Ok;
  }
 
-#else /* REENTRANT */
+#else /* TT_CONFIG_OPTION_THREAD_SAFE */
 
   /*******************************************************************/
   /*******************************************************************/
@@ -255,7 +260,8 @@
  *
  ******************************************************************/
 
- TT_Error TTFile_Init( PEngine_Instance  engine )
+ LOCAL_FUNC
+ TT_Error  TTFile_Init( PEngine_Instance  engine )
  {
    engine.file_component = NULL;
 
@@ -271,6 +277,7 @@
  *
  ******************************************************************/
 
+ LOCAL_FUNC
  TT_Error  TTFile_Done( PEngine_Instance  engine )
  {
    return TT_Err_Ok;
@@ -291,6 +298,7 @@
  *
  ******************************************************************/
 
+  EXPORT_FUNC
   TT_Error  TT_Use_Stream( TT_Stream   input_stream,
                            TT_Stream*  copy )
   {
@@ -329,12 +337,13 @@
  *
  ******************************************************************/
 
+ EXPORT_FUNC
  TT_Error  TT_Done_Stream( TT_Stream*  stream )
  {
    return TT_Close_Stream( stream );
  }
 
-#endif /* REENTRANT */
+#endif /* TT_CONFIG_OPTION_THREAD_SAFE */
 
 
   /*******************************************************************/
@@ -392,7 +401,7 @@
     map->refcount--;
     if ( map->refcount <= 0 )
     {
-      munmap ( (char*)map->base, map->size );
+      munmap ( (void*)map->base, map->size );
       FREE( map );
     }
   }
@@ -409,16 +418,17 @@
  *            filepathname   pathname of the file to open
  *            stream         address of target TT_Stream structure
  *
- *  Output :  SUCCESS on sucess, FAILURE on error.
+ *  Output :  SUCCESS on success, FAILURE on error.
  *            The target stream is set to -1 in case of failure.
  *
  ******************************************************************/
 
-  TT_Error  TT_Open_Stream( const char*  filepathname, 
-                            TT_Stream*   stream )
+  LOCAL_FUNC
+  TT_Error  TT_Open_Stream( const String*  filepathname, 
+                            TT_Stream*     stream )
   {
     TT_Error     error;
-    int          file;
+    Int          file;
     PStream_Rec  stream_rec;
     TFileMap*    map;
     struct stat  stat_buf;
@@ -436,7 +446,7 @@
 
     stream_rec = STREAM2REC( *stream );
 
-    file = open( filepathname, O_RDONLY );
+    file = open( (String*)filepathname, O_RDONLY );
     if ( file < 0 )
       goto File_Fail;
 
@@ -460,7 +470,7 @@
     stream_rec->map = map;
     stream_rec->pos = 0;
 
-#ifndef TT_CONFIG_REENTRANT
+#ifndef TT_CONFIG_OPTION_THREAD_SAFE
     CUR_Stream = stream_rec;
 #endif
 
@@ -491,9 +501,10 @@
  *
  ******************************************************************/
 
+  LOCAL_FUNC
   TT_Error  TT_Close_Stream( TT_Stream*  stream )
   {
-    PStream_Rec  rec = STREAM2REC(*stream);
+    PStream_Rec  rec = STREAM2REC( *stream );
 
 
     Release_Map( rec->map );
@@ -520,6 +531,7 @@
  *
  ******************************************************************/
  
+  EXPORT_FUNC
   TT_Error  TT_Flush_Stream( TT_Stream*  stream )
   {
     /* XXX - DUMMY IMPLEMENTATION */
@@ -536,10 +548,11 @@
  *
  *  Input  :  stream     the stream
  *
- *  Output :  lenght of stream in bytes.            
+ *  Output :  Length of stream in bytes.            
  *
  ******************************************************************/
 
+  EXPORT_FUNC
   Long  TT_Stream_Size( TT_Stream  stream )
   {
     PStream_Rec  rec = STREAM2REC( stream );
@@ -560,11 +573,12 @@
  *
  *  Input  :  position     new position in file
  *
- *  Output :  SUCCESS on success. FAILURE if out of range.
+ *  Output :  SUCCESS on success.  FAILURE if out of range.
  *
  ******************************************************************/
 
-  TT_Error  TT_Seek_File( STREAM_ARGS long position )
+  EXPORT_FUNC
+  TT_Error  TT_Seek_File( STREAM_ARGS Long  position )
   {
     if ( position > CUR_Stream->map->size )
       return TT_Err_Invalid_File_Offset;
@@ -587,7 +601,8 @@
  *
  ******************************************************************/
 
-  TT_Error  TT_Skip_File( STREAM_ARGS long distance )
+  EXPORT_FUNC
+  TT_Error  TT_Skip_File( STREAM_ARGS Long  distance )
   {
     return TT_Seek_File( STREAM_VARS CUR_Stream->pos + distance );
   }
@@ -602,11 +617,12 @@
  *  Input  :  buffer    target buffer
  *            count     length in bytes to read
  *
- *  Output :  SUCCESS on success. FAILURE if out of range.
+ *  Output :  SUCCESS on success.  FAILURE if out of range.
  *
  ******************************************************************/
 
-  TT_Error  TT_Read_File( STREAM_ARGS void* buffer, long count )
+  EXPORT_FUNC
+  TT_Error  TT_Read_File( STREAM_ARGS void*  buffer, Long  count )
   {
     if ( CUR_Stream->pos + count > CUR_Stream->map->size )
       return TT_Err_Invalid_File_Read;
@@ -633,9 +649,10 @@
  *
  ******************************************************************/
 
-  TT_Error  TT_Read_At_File( STREAM_ARGS long   position, 
-                                         void*  buffer, 
-                                         long   count )
+  EXPORT_FUNC
+  TT_Error  TT_Read_At_File( STREAM_ARGS Long   position,
+                                         void*  buffer,
+                                         Long   count )
   {
     TT_Error  error;
 
@@ -660,6 +677,7 @@
  *
  ******************************************************************/
 
+  EXPORT_FUNC
   Long  TT_File_Pos( STREAM_ARG )
   {
     return CUR_Stream->pos;
@@ -687,7 +705,8 @@
  *
  ******************************************************************/
 
-  TT_Error  TT_Access_Frame( STREAM_ARGS FRAME_ARGS int  size )
+  EXPORT_FUNC
+  TT_Error  TT_Access_Frame( STREAM_ARGS FRAME_ARGS Long  size )
   {
     if ( CUR_Frame.address != NULL )
       return TT_Err_Nested_Frame_Access;
@@ -736,7 +755,8 @@
  *
  ******************************************************************/
 
-  TT_Error  TT_Check_And_Access_Frame( STREAM_ARGS FRAME_ARGS int  size )
+  EXPORT_FUNC
+  TT_Error  TT_Check_And_Access_Frame( STREAM_ARGS FRAME_ARGS Long  size )
   {
     TT_Error  error;
     Long      readBytes;
@@ -782,18 +802,20 @@
  *
  ******************************************************************/
 
+  EXPORT_FUNC
   TT_Error  TT_Forget_Frame( FRAME_ARG )
   {
     if ( CUR_Frame.address == NULL )
       return TT_Err_Nested_Frame_Access;
 
     /* If we were using a duplicate in case of overflow, free it now */
-    if ( CUR_Frame.address < (unsigned char *) CUR_Stream->map->base ||
-         CUR_Frame.address >= (unsigned char *) CUR_Stream->map->base +
+    if ( CUR_Frame.address < (Byte*)CUR_Stream->map->base ||
+         CUR_Frame.address >= (Byte*)CUR_Stream->map->base +
          CUR_Stream->map->size )
       FREE( CUR_Frame.address );
 
     ZERO_Frame( files.frame );
+
     return TT_Err_Ok;
   }
 
@@ -815,13 +837,18 @@
  *
  ******************************************************************/
 
-  unsigned char  TT_Get_Byte( FRAME_ARG )
+#if 0
+
+  EXPORT_FUNC
+  Byte  TT_Get_Byte( FRAME_ARG )
   {
     CHECK_FRAME( CUR_Frame, 1 );
 
-    return (unsigned char)(*CUR_Frame.cursor++);
+    return (Byte)(*CUR_Frame.cursor++);
   }
 
+#endif
+
 
 /*******************************************************************
  *
@@ -840,11 +867,12 @@
  *
  ******************************************************************/
 
-  char  TT_Get_Char( FRAME_ARG )
+  EXPORT_FUNC
+  Char  TT_Get_Char( FRAME_ARG )
   {
     CHECK_FRAME( CUR_Frame, 1 );
 
-    return (char)(*CUR_Frame.cursor++);
+    return (Char)(*CUR_Frame.cursor++);
   }
 
 
@@ -865,15 +893,16 @@
  *
  ******************************************************************/
 
-  short  TT_Get_Short( FRAME_ARG )
+  EXPORT_FUNC
+  Short  TT_Get_Short( FRAME_ARG )
   {
-    short  getshort;
+    Short  getshort;
 
 
     CHECK_FRAME( CUR_Frame, 2 );
 
-    getshort = ((short)CUR_Frame.cursor[0] << 8) |
-                (short)CUR_Frame.cursor[1];
+    getshort = ((Short)CUR_Frame.cursor[0] << 8) |
+                (Short)CUR_Frame.cursor[1];
 
     CUR_Frame.cursor += 2;
 
@@ -898,21 +927,26 @@
  *
  ******************************************************************/
 
-  unsigned short  TT_Get_UShort( FRAME_ARG )
+#if 0
+
+  EXPORT_FUNC
+  UShort  TT_Get_UShort( FRAME_ARG )
   {
-    unsigned short  getshort;
+    UShort  getshort;
 
 
     CHECK_FRAME( CUR_Frame, 2 );
 
-    getshort = ((unsigned short)CUR_Frame.cursor[0] << 8) |
-                (unsigned short)CUR_Frame.cursor[1];
+    getshort = ((UShort)CUR_Frame.cursor[0] << 8) |
+                (UShort)CUR_Frame.cursor[1];
 
     CUR_Frame.cursor += 2;
 
     return getshort;
   }
 
+#endif
+
 
 /*******************************************************************
  *
@@ -931,17 +965,18 @@
  *
  ******************************************************************/
 
-  long  TT_Get_Long( FRAME_ARG )
+  EXPORT_FUNC
+  Long  TT_Get_Long( FRAME_ARG )
   {
-    long  getlong;
+    Long  getlong;
 
 
     CHECK_FRAME( CUR_Frame, 4 );
 
-    getlong = ((long)CUR_Frame.cursor[0] << 24) |
-              ((long)CUR_Frame.cursor[1] << 16) |
-              ((long)CUR_Frame.cursor[2] << 8 ) |
-               (long)CUR_Frame.cursor[3];
+    getlong = ((Long)CUR_Frame.cursor[0] << 24) |
+              ((Long)CUR_Frame.cursor[1] << 16) |
+              ((Long)CUR_Frame.cursor[2] << 8 ) |
+               (Long)CUR_Frame.cursor[3];
 
     CUR_Frame.cursor += 4;
 
@@ -966,22 +1001,27 @@
  *
  ******************************************************************/
 
-  unsigned long  TT_Get_ULong( FRAME_ARG )
+#if 0
+
+  EXPORT_FUNC
+  ULong  TT_Get_ULong( FRAME_ARG )
   {
-    unsigned long  getlong;
+    ULong  getlong;
 
 
     CHECK_FRAME( CUR_Frame, 4 );
 
-    getlong = ( ((unsigned long)CUR_Frame.cursor[0] << 24) |
-                ((unsigned long)CUR_Frame.cursor[1] << 16) |
-                ((unsigned long)CUR_Frame.cursor[2] << 8 ) |
-                 (unsigned long)CUR_Frame.cursor[3] );
+    getlong = ( ((ULong)CUR_Frame.cursor[0] << 24) |
+                ((ULong)CUR_Frame.cursor[1] << 16) |
+                ((ULong)CUR_Frame.cursor[2] << 8 ) |
+                 (ULong)CUR_Frame.cursor[3] );
 
     CUR_Frame.cursor += 4;
 
     return getlong;
   }
 
+#endif
+
 
 /* END */
