@@ -29,6 +29,71 @@
 #define GSUB_ID  Build_Extension_ID( 'G', 'S', 'U', 'B' )
 
 
+#define ADD_STRING( in, num_in, out, num_out, data )        \
+          ( ( error = Add_String( (in), (num_in),           \
+                                  (out), (num_out),         \
+                                  (data) ) ) != TT_Err_Ok )
+
+#define String_Send_Begin( s )     (s)->pos = 0;
+#define String_Send_End( s )       (s)->pos = 0;
+#define String_Receive_Begin( s )  (s)->pos = 0;
+#define String_Receive_End( s )    (s)->length = (s)->pos; \
+                                   (s)->pos    = 0;
+
+
+
+  /**********************
+   * Auxiliary functions
+   **********************/
+
+
+  /* The following function copies `num_out' elements from `data' to
+     `out', advancing the array pointer in the `in' structure by `num_in'
+     elements and in `out' by `num_out' elements.  If the string array
+     in `out' is empty or too small, it allocates resp. reallocates the
+     string array.                                                        */
+
+  static TT_Error  Add_String( TTO_String*  in,
+                               UShort       num_in,
+                               TTO_String*  out,
+                               UShort       num_out,
+                               UShort*      data )
+  {
+    TT_Error  error = TT_Err_Ok;
+
+
+    /* sanity check */
+
+    if ( !in || !out ||
+         in->length == 0 || in->pos >= in->length ||
+         in->length >= in->pos + num_in )
+      return TT_Err_Invalid_Argument;
+
+    if ( out->allocated == 0 )
+    {
+      if ( ALLOC( out->string, in->length - ( in->pos + num_in ) ) )
+        return error;
+      out->allocated = in->length - ( in->pos + num_in );
+    }
+    else if ( out->pos + num_out >= out->allocated )
+    {
+      if ( REALLOC( out->string, out->pos + num_out +
+                                 in->length - ( in->pos + num_in ) ) )
+        return error;
+      out->allocated = out->pos + num_out +
+                       in->length - ( in->pos + num_in );
+    }
+
+    MEM_Copy( &out->string[in->pos], data, num_out * sizeof ( UShort ) );
+
+    in->pos  += num_in;
+    out->pos += num_out;
+
+    return TT_Err_Ok;
+  }
+
+
+
   /**********************
    * Extension Functions
    **********************/
@@ -47,6 +112,9 @@
     if ( !gsub )
       return TT_Err_Ok;
 
+    /* a null offset indicates that there is no GSUB table */
+    gsub->offset = 0;
+
     /* we store the start offset and the size of the subtable */
     table = TT_LookUp_Table( face, TTAG_GSUB );
     if ( table < 0 )
@@ -79,7 +147,7 @@
 
     if ( gsub->loaded )
     {
-      Free_LookupList( &gsub->LookupList, Load_GSUB );
+      Free_LookupList( &gsub->LookupList, GSUB );
       Free_FeatureList( &gsub->FeatureList );
       Free_ScriptList( &gsub->ScriptList );
     }
@@ -123,6 +191,9 @@
     if ( error )
       return error;
 
+    if ( gsub->offset == 0 )
+      return TT_Err_Ok;             /* no GSUB table; nothing to do */
+
     /* now access stream */
     if ( USE_Stream( faze->stream, stream ) )
       return error;
@@ -142,7 +213,7 @@
 
     if ( FILE_Seek( gsub->offset + 4L ) ||
          ( error = Load_LookupList( &gsub->LookupList,
-                                    faze, Load_GSUB ) ) != TT_Err_Ok )
+                                    faze, GSUB ) ) != TT_Err_Ok )
       goto Fail3;
 
     gsub->loaded = TRUE;
@@ -152,7 +223,7 @@
 
 
   Fail3:
-    Free_LookupList( &gsub->LookupList, Load_GSUB );
+    Free_LookupList( &gsub->LookupList, GSUB );
 
   Fail2:
     Free_FeatureList( &gsub->FeatureList );
@@ -183,8 +254,10 @@
   {
     DEFINE_LOAD_LOCALS( input->stream );
 
-    UShort  n, count;
-    ULong   cur_offset, new_offset, base_offset;
+    UShort   n, count;
+    ULong    cur_offset, new_offset, base_offset;
+
+    UShort*  s;
 
 
     base_offset = FILE_Pos();
@@ -228,11 +301,13 @@
       if ( ALLOC_ARRAY( ss->ssf.ssf2.Substitute, count, TT_UShort ) )
         return error;
 
+      s = ss->ssf.ssf2.Substitute;
+
       if ( ACCESS_Frame( count * 2L ) )
         return error;
 
       for ( n = 0; n < count; n++ )
-        ss->ssf.ssf2.Substitute[n] = GET_UShort();
+        s[n] = GET_UShort();
 
       FORGET_Frame();
 
@@ -264,6 +339,49 @@
   }
 
 
+  static TT_Error  Lookup_Single( TTO_SingleSubst*  ss,
+                                  TTO_String*       in,
+                                  TTO_String*       out )
+  {
+    UShort    index, value[1];
+    TT_Error  error;
+
+
+    /* sanity check */
+
+    if ( !in || !out || in->length == 0 || in->pos >= in->length )
+      return TT_Err_Invalid_Argument;
+
+    error = Coverage_Index( &ss->Coverage, in->string[in->pos], &index );
+    if ( error == TTO_Err_Not_Covered )
+    {
+      if ( ADD_STRING( in, 1, out, 1, &in->string[in->pos] ) )
+        return error;
+    }
+    else
+    {
+      switch ( ss->SubstFormat)
+      {
+      case 1:
+        value[0] = in->string[in->pos] + ss->ssf.ssf1.DeltaGlyphID;
+        if ( ADD_STRING( in, 1, out, 1, value ) )
+          return error;
+        break;
+      case 2:
+        if ( index >= ss->ssf.ssf2.GlyphCount )
+          return TTO_Err_Invalid_GSUB_SubTable;
+        if ( ADD_STRING( in, 1, out, 1, &ss->ssf.ssf2.Substitute[index] ) )
+          return error;
+        break;
+      default:
+        return TTO_Err_Invalid_GSUB_SubTable;
+      }
+    }
+
+    return TT_Err_Ok;
+  }
+
+
   /* LookupType 2 */
 
   /* Sequence */
@@ -273,7 +391,9 @@
   {
     DEFINE_LOAD_LOCALS( input->stream );
 
-    UShort  n, count;
+    UShort   n, count;
+
+    UShort*  sub;
 
 
     if ( ACCESS_Frame( 2L ) )
@@ -288,11 +408,13 @@
     if ( ALLOC_ARRAY( s->Substitute, count, TT_UShort ) )
       return error;
 
+    sub = s->Substitute;
+
     if ( ACCESS_Frame ( count * 2L ) )
       return error;
 
     for ( n = 0; n < count; n++ )
-      s->Substitute[n] = GET_UShort();
+      sub[n] = GET_UShort();
 
     FORGET_Frame();
 
@@ -314,8 +436,10 @@
   {
     DEFINE_LOAD_LOCALS( input->stream );
 
-    UShort  n, count;
-    ULong   cur_offset, new_offset, base_offset;
+    UShort         n, count;
+    ULong          cur_offset, new_offset, base_offset;
+
+    TTO_Sequence*  s;
 
 
     base_offset = FILE_Pos();
@@ -346,6 +470,8 @@
     if ( ALLOC_ARRAY( ms->Sequence, count, TTO_Sequence ) )
       return error;
 
+    s = ms->Sequence;
+
     for ( n = 0; n < count; n++ )
     {
       if ( ACCESS_Frame( 2L ) )
@@ -357,7 +483,7 @@
 
       cur_offset = FILE_Pos();
       if ( FILE_Seek( new_offset ) ||
-           ( error = Load_Sequence( &ms->Sequence[n], input ) ) != TT_Err_Ok )
+           ( error = Load_Sequence( &s[n], input ) ) != TT_Err_Ok )
         return error;
       (void)FILE_Seek( cur_offset );
     }
@@ -394,7 +520,9 @@
   {
     DEFINE_LOAD_LOCALS( input->stream );
 
-    UShort  n, count;
+    UShort   n, count;
+
+    UShort*  a;
 
 
     if ( ACCESS_Frame( 2L ) )
@@ -409,11 +537,13 @@
     if ( ALLOC_ARRAY( as->Alternate, count, TT_UShort ) )
       return error;
 
+    a = as->Alternate;
+
     if ( ACCESS_Frame( count * 2L ) )
       return error;
 
     for ( n = 0; n < count; n++ )
-      as->Alternate[n] = GET_UShort();
+      a[n] = GET_UShort();
 
     FORGET_Frame();
 
@@ -435,8 +565,10 @@
   {
     DEFINE_LOAD_LOCALS( input->stream );
 
-    UShort  n, count;
-    ULong   cur_offset, new_offset, base_offset;
+    UShort             n, count;
+    ULong              cur_offset, new_offset, base_offset;
+
+    TTO_AlternateSet*  aset;
 
 
     base_offset = FILE_Pos();
@@ -467,6 +599,8 @@
     if ( ALLOC_ARRAY( as->AlternateSet, count, TTO_AlternateSet ) )
       return error;
 
+    aset = as->AlternateSet;
+
     for ( n = 0; n < count; n++ )
     {
       if ( ACCESS_Frame( 2L ) )
@@ -478,8 +612,7 @@
 
       cur_offset = FILE_Pos();
       if ( FILE_Seek( new_offset ) ||
-           ( error = Load_AlternateSet( &as->AlternateSet[n],
-                                        input ) ) != TT_Err_Ok )
+           ( error = Load_AlternateSet( &aset[n], input ) ) != TT_Err_Ok )
         return error;
       (void)FILE_Seek( cur_offset );
     }
@@ -516,7 +649,9 @@
   {
     DEFINE_LOAD_LOCALS( input->stream );
 
-    UShort  n, count;
+    UShort   n, count;
+
+    UShort*  c;
 
 
     if ( ACCESS_Frame( 4L ) )
@@ -534,11 +669,13 @@
     if ( ALLOC_ARRAY( l->Component, count, TT_UShort ) )
       return error;
 
+    c = l->Component;
+
     if ( ACCESS_Frame( count * 2L ) )
       return error;
 
     for ( n = 0; n < count; n++ )
-      l->Component[n] = GET_UShort();
+      c[n] = GET_UShort();
 
     FORGET_Frame();
 
@@ -560,8 +697,10 @@
   {
     DEFINE_LOAD_LOCALS( input->stream );
 
-    UShort  n, count;
-    ULong   cur_offset, new_offset, base_offset;
+    UShort         n, count;
+    ULong          cur_offset, new_offset, base_offset;
+
+    TTO_Ligature*  l;
 
 
     base_offset = FILE_Pos();
@@ -578,6 +717,8 @@
     if ( ALLOC_ARRAY( ls->Ligature, count, TTO_Ligature ) )
       return error;
 
+    l = ls->Ligature;
+
     for ( n = 0; n < count; n++ )
     {
       if ( ACCESS_Frame( 2L ) )
@@ -589,7 +730,7 @@
 
       cur_offset = FILE_Pos();
       if ( FILE_Seek( new_offset ) ||
-           ( error = Load_Ligature( &ls->Ligature[n], input ) ) != TT_Err_Ok )
+           ( error = Load_Ligature( &l[n], input ) ) != TT_Err_Ok )
         return error;
       (void)FILE_Seek( cur_offset );
     }
@@ -622,8 +763,10 @@
   {
     DEFINE_LOAD_LOCALS( input->stream );
 
-    UShort  n, count;
-    ULong   cur_offset, new_offset, base_offset;
+    UShort            n, count;
+    ULong             cur_offset, new_offset, base_offset;
+
+    TTO_LigatureSet*  lset;
 
 
     base_offset = FILE_Pos();
@@ -654,6 +797,8 @@
     if ( ALLOC_ARRAY( ls->LigatureSet, count, TTO_LigatureSet ) )
       return error;
 
+    lset = ls->LigatureSet;
+
     for ( n = 0; n < count; n++ )
     {
       if ( ACCESS_Frame( 2L ) )
@@ -665,8 +810,7 @@
 
       cur_offset = FILE_Pos();
       if ( FILE_Seek( new_offset ) ||
-           ( error = Load_LigatureSet( &ls->LigatureSet[n],
-                                       input ) ) != TT_Err_Ok )
+           ( error = Load_LigatureSet( &lset[n], input ) ) != TT_Err_Ok )
         return error;
       (void)FILE_Seek( cur_offset );
     }
@@ -703,7 +847,10 @@
   {
     DEFINE_LOAD_LOCALS( input->stream );
 
-    UShort  n, count;
+    UShort                  n, count;
+
+    UShort*                 i;
+    TTO_SubstLookupRecord*  slr;
 
 
     if ( ACCESS_Frame( 4L ) )
@@ -721,11 +868,13 @@
     if ( ALLOC_ARRAY( sr->Input, count, TT_UShort ) )
       return error;
 
+    i = sr->Input;
+
     if ( ACCESS_Frame( count * 2L ) )
       return error;
 
     for ( n = 0; n < count; n++ )
-      sr->Input[n] = GET_UShort();
+      i[n] = GET_UShort();
 
     FORGET_Frame();
 
@@ -736,13 +885,15 @@
     if ( ALLOC_ARRAY( sr->SubstLookupRecord, count, TTO_SubstLookupRecord ) )
       return error;
 
+    slr = sr->SubstLookupRecord;
+
     if ( ACCESS_Frame( count * 4L ) )
       return error;
 
     for ( n = 0; n < count; n++ )
     {
-      sr->SubstLookupRecord[n].SequenceIndex   = GET_UShort();
-      sr->SubstLookupRecord[n].LookupListIndex = GET_UShort();
+      slr[n].SequenceIndex   = GET_UShort();
+      slr[n].LookupListIndex = GET_UShort();
     }
 
     FORGET_Frame();
@@ -767,8 +918,10 @@
   {
     DEFINE_LOAD_LOCALS( input->stream );
 
-    UShort  n, count;
-    ULong   cur_offset, new_offset, base_offset;
+    UShort        n, count;
+    ULong         cur_offset, new_offset, base_offset;
+
+    TTO_SubRule*  sr;
 
 
     base_offset = FILE_Pos();
@@ -785,6 +938,8 @@
     if ( ALLOC_ARRAY( srs->SubRule, count, TTO_SubRule ) )
       return error;
 
+    sr = srs->SubRule;
+
     for ( n = 0; n < count; n++ )
     {
       if ( ACCESS_Frame( 2L ) )
@@ -796,7 +951,7 @@
 
       cur_offset = FILE_Pos();
       if ( FILE_Seek( new_offset ) ||
-           ( error = Load_SubRule( &srs->SubRule[n], input ) ) != TT_Err_Ok )
+           ( error = Load_SubRule( &sr[n], input ) ) != TT_Err_Ok )
         return error;
       (void)FILE_Seek( cur_offset );
     }
@@ -829,8 +984,10 @@
   {
     DEFINE_LOAD_LOCALS( input->stream );
 
-    UShort  n, count;
-    ULong   cur_offset, new_offset, base_offset;
+    UShort           n, count;
+    ULong            cur_offset, new_offset, base_offset;
+
+    TTO_SubRuleSet*  srs;
 
 
     base_offset = FILE_Pos();
@@ -860,6 +1017,8 @@
     if ( ALLOC_ARRAY( csf1->SubRuleSet, count, TTO_SubRuleSet ) )
       return error;
 
+    srs = csf1->SubRuleSet;
+
     for ( n = 0; n < count; n++ )
     {
       if ( ACCESS_Frame( 2L ) )
@@ -871,8 +1030,7 @@
 
       cur_offset = FILE_Pos();
       if ( FILE_Seek( new_offset ) ||
-           ( error = Load_SubRuleSet( &csf1->SubRuleSet[n],
-                                      input ) ) != TT_Err_Ok )
+           ( error = Load_SubRuleSet( &srs[n], input ) ) != TT_Err_Ok )
         return error;
       (void)FILE_Seek( cur_offset );
     }
@@ -907,7 +1065,10 @@
   {
     DEFINE_LOAD_LOCALS( input->stream );
 
-    UShort  n, count;
+    UShort                  n, count;
+
+    UShort*                 c;
+    TTO_SubstLookupRecord*  slr;
 
 
     if ( ACCESS_Frame( 4L ) )
@@ -925,11 +1086,13 @@
     if ( ALLOC_ARRAY( scr->Class, count, TT_UShort ) )
       return error;
 
+    c = scr->Class;
+
     if ( ACCESS_Frame( count * 2L ) )
       return error;
 
     for ( n = 0; n < count; n++ )
-      scr->Class[n] = GET_UShort();
+      c[n] = GET_UShort();
 
     FORGET_Frame();
 
@@ -940,13 +1103,15 @@
     if ( ALLOC_ARRAY( scr->SubstLookupRecord, count, TTO_SubstLookupRecord ) )
       return error;
 
+    slr = scr->SubstLookupRecord;
+
     if ( ACCESS_Frame( count * 4L ) )
       return error;
 
     for ( n = 0; n < count; n++ )
     {
-      scr->SubstLookupRecord[n].SequenceIndex   = GET_UShort();
-      scr->SubstLookupRecord[n].LookupListIndex = GET_UShort();
+      slr[n].SequenceIndex   = GET_UShort();
+      slr[n].LookupListIndex = GET_UShort();
     }
 
     FORGET_Frame();
@@ -971,8 +1136,10 @@
   {
     DEFINE_LOAD_LOCALS( input->stream );
 
-    UShort  n, count;
-    ULong   cur_offset, new_offset, base_offset;
+    UShort             n, count;
+    ULong              cur_offset, new_offset, base_offset;
+
+    TTO_SubClassRule*  scr;
 
 
     base_offset = FILE_Pos();
@@ -989,6 +1156,8 @@
     if ( ALLOC_ARRAY( scs->SubClassRule, count, TTO_SubClassRule ) )
       return error;
 
+    scr = scs->SubClassRule;
+
     for ( n = 0; n < count; n++ )
     {
       if ( ACCESS_Frame( 2L ) )
@@ -1000,8 +1169,7 @@
 
       cur_offset = FILE_Pos();
       if ( FILE_Seek( new_offset ) ||
-           ( error = Load_SubClassRule( &scs->SubClassRule[n],
-                                        input ) ) != TT_Err_Ok )
+           ( error = Load_SubClassRule( &scr[n], input ) ) != TT_Err_Ok )
         return error;
       (void)FILE_Seek( cur_offset );
     }
@@ -1034,8 +1202,10 @@
   {
     DEFINE_LOAD_LOCALS( input->stream );
 
-    UShort  n, count;
-    ULong   cur_offset, new_offset, base_offset;
+    UShort            n, count;
+    ULong             cur_offset, new_offset, base_offset;
+
+    TTO_SubClassSet*  scs;
 
 
     base_offset = FILE_Pos();
@@ -1079,6 +1249,8 @@
     if ( ALLOC_ARRAY( csf2->SubClassSet, count, TTO_SubClassSet ) )
       return error;
 
+    scs = csf2->SubClassSet;
+
     for ( n = 0; n < count; n++ )
     {
       if ( ACCESS_Frame( 2L ) )
@@ -1092,8 +1264,7 @@
       {
         cur_offset = FILE_Pos();
         if ( FILE_Seek( new_offset ) ||
-             ( error = Load_SubClassSet( &csf2->SubClassSet[n],
-                                         input ) ) != TT_Err_Ok )
+             ( error = Load_SubClassSet( &scs[n], input ) ) != TT_Err_Ok )
           return error;
         (void)FILE_Seek( cur_offset );
       }
@@ -1136,8 +1307,11 @@
   {
     DEFINE_LOAD_LOCALS( input->stream );
 
-    UShort  n, count;
-    ULong   cur_offset, new_offset, base_offset;
+    UShort                  n, count;
+    ULong                   cur_offset, new_offset, base_offset;
+
+    TTO_Coverage*           c;
+    TTO_SubstLookupRecord*  slr;
 
 
     base_offset = FILE_Pos();
@@ -1158,12 +1332,13 @@
     if ( ALLOC_ARRAY( csf3->Coverage, count, TTO_Coverage ) )
       return error;
 
+    c = csf3->Coverage;
+
     for ( n = 0; n < count; n++ )
     {
       cur_offset = FILE_Pos();
       if ( FILE_Seek( new_offset ) ||
-           ( error = Load_Coverage( &csf3->Coverage[n],
-                                    input ) ) != TT_Err_Ok )
+           ( error = Load_Coverage( &c[n], input ) ) != TT_Err_Ok )
         return error;
       (void)FILE_Seek( cur_offset );
     }
@@ -1176,13 +1351,15 @@
                       TTO_SubstLookupRecord ) )
       return error;
 
+    slr = csf3->SubstLookupRecord;
+
     if ( ACCESS_Frame( count * 4L ) )
       return error;
 
     for ( n = 0; n < count; n++ )
     {
-      csf3->SubstLookupRecord[n].SequenceIndex   = GET_UShort();
-      csf3->SubstLookupRecord[n].LookupListIndex = GET_UShort();
+      slr[n].SequenceIndex   = GET_UShort();
+      slr[n].LookupListIndex = GET_UShort();
     }
 
     FORGET_Frame();
@@ -1261,4 +1438,234 @@
   }
 
 
+  /***********
+   * GSUB API
+   ***********/
+
+
+  TT_Error  TT_GSUB_Select_Script( TTO_GSUBHeader*  gsub,
+                                   TT_ULong         script_tag,
+                                   TT_UShort*       script_index )
+  {
+    UShort             n;
+
+    TTO_ScriptList*    sl = &gsub->ScriptList;
+    TTO_ScriptRecord*  sr = sl->ScriptRecord;
+
+
+    for ( n = 0; n < sl->ScriptCount; n++ )
+      if ( script_tag == sr[n].ScriptTag )
+      {
+        *script_index = n;
+        return TT_Err_Ok;
+      }
+
+    return TTO_Err_Not_Covered;
+  }
+
+
+  TT_Error  TT_GSUB_Select_Language( TTO_GSUBHeader*  gsub,
+                                     TT_ULong         language_tag,
+                                     TT_UShort        script_index,
+                                     TT_UShort*       language_index )
+  {
+    UShort              n;
+
+    TTO_ScriptList*     sl = &gsub->ScriptList;
+    TTO_ScriptRecord*   sr = sl->ScriptRecord;
+    TTO_Script*         s;
+    TTO_LangSysRecord*  lsr;
+
+
+    if ( script_index >= sl->ScriptCount )
+      return TT_Err_Invalid_Argument;
+
+    s   = &sr[script_index].Script;
+    lsr = s->LangSysRecord;
+
+    for ( n = 0; n < s->LangSysCount; n++ )
+      if ( language_tag == lsr[n].LangSysTag )
+      {
+        *language_index = n;
+        return TT_Err_Ok;
+      }
+
+    return TTO_Err_Not_Covered;
+  }
+
+
+  /* selecting 0xFFFF for language_index asks for the values of the
+     default language (DefaultLangSys)                              */
+
+  TT_Error  TT_GSUB_Select_Feature( TTO_GSUBHeader*  gsub,
+                                    TT_ULong         feature_tag,
+                                    TT_UShort        script_index,
+                                    TT_UShort        language_index,
+                                    TT_UShort*       feature_index )
+  {
+    UShort              n;
+
+    TTO_ScriptList*     sl = &gsub->ScriptList;
+    TTO_ScriptRecord*   sr = sl->ScriptRecord;
+    TTO_Script*         s;
+    TTO_LangSysRecord*  lsr;
+    TTO_LangSys*        ls;
+    UShort*             fi;
+
+    TTO_FeatureList*    fl = &gsub->FeatureList;
+    TTO_FeatureRecord*  fr = fl->FeatureRecord;
+
+
+    if ( script_index >= sl->ScriptCount )
+      return TT_Err_Invalid_Argument;
+
+    s   = &sr[script_index].Script;
+    lsr = s->LangSysRecord;
+
+    if ( language_index == 0xFFFF )
+      ls = &s->DefaultLangSys;
+    else
+    {
+      if ( language_index >= s->LangSysCount )
+        return TT_Err_Invalid_Argument;
+
+      ls = &lsr[language_index].LangSys;
+    }
+    fi = ls->FeatureIndex;
+
+    for ( n = 0; n < ls->FeatureCount; n++ )
+    {
+      if ( fi[n] >= fl->FeatureCount )
+        return TTO_Err_Invalid_GSUB_SubTable_Format;
+
+      if ( feature_tag == fr[fi[n]].FeatureTag )
+      {
+        *feature_index = n;
+        return TT_Err_Ok;
+      }
+    }
+
+    return TTO_Err_Not_Covered;
+  }
+
+
+  /* The next three functions return a null-terminated list */
+
+  TT_Error  TT_GSUB_Query_Scripts( TTO_GSUBHeader*  gsub,
+                                   TT_ULong**       script_tag_list )
+  {
+    UShort             n;
+    TT_Error           error;
+    ULong*             stl;
+
+    TTO_ScriptList*    sl = &gsub->ScriptList;
+    TTO_ScriptRecord*  sr = sl->ScriptRecord;
+
+
+    if ( ALLOC_ARRAY( stl, sl->ScriptCount + 1, TT_ULong ) )
+      return error;
+
+    for ( n = 0; n < sl->ScriptCount; n++ )
+      stl[n] = sr[n].ScriptTag;
+    stl[n] = 0;
+
+    *script_tag_list = stl;
+
+    return TT_Err_Ok;
+  }
+
+
+  TT_Error  TT_GSUB_Query_Languages( TTO_GSUBHeader*  gsub,
+                                     TT_UShort        script_index,
+                                     TT_ULong**       language_tag_list )
+  {
+    UShort              n;
+    TT_Error            error;
+    ULong*              ltl;
+
+    TTO_ScriptList*     sl = &gsub->ScriptList;
+    TTO_ScriptRecord*   sr = sl->ScriptRecord;
+    TTO_Script*         s;
+    TTO_LangSysRecord*  lsr;
+
+
+    if ( script_index >= sl->ScriptCount )
+      return TT_Err_Invalid_Argument;
+
+    s   = &sr[script_index].Script;
+    lsr = s->LangSysRecord;
+
+    if ( ALLOC_ARRAY( ltl, s->LangSysCount + 1, TT_ULong ) )
+      return error;
+
+    for ( n = 0; n < s->LangSysCount; n++ )
+      ltl[n] = lsr[n].LangSysTag;
+    ltl[n] = 0;
+
+    *language_tag_list = ltl;
+
+    return TT_Err_Ok;
+  }
+
+
+  /* selecting 0xFFFF for language_index asks for the values of the
+     default language (DefaultLangSys)                              */
+
+  TT_Error  TT_GSUB_Query_Features( TTO_GSUBHeader*  gsub,
+                                    TT_UShort        script_index,
+                                    TT_UShort        language_index,
+                                    TT_ULong**       feature_tag_list )
+  {
+    UShort              n;
+    TT_Error            error;
+    ULong*              ftl;
+
+    TTO_ScriptList*     sl = &gsub->ScriptList;
+    TTO_ScriptRecord*   sr = sl->ScriptRecord;
+    TTO_Script*         s;
+    TTO_LangSysRecord*  lsr;
+    TTO_LangSys*        ls;
+    UShort*             fi;
+
+    TTO_FeatureList*    fl = &gsub->FeatureList;
+    TTO_FeatureRecord*  fr = fl->FeatureRecord;
+
+
+    if ( script_index >= sl->ScriptCount )
+      return TT_Err_Invalid_Argument;
+
+    s   = &sr[script_index].Script;
+    lsr = s->LangSysRecord;
+
+    if ( language_index == 0xFFFF )
+      ls = &s->DefaultLangSys;
+    else
+    {
+      if ( language_index >= s->LangSysCount )
+        return TT_Err_Invalid_Argument;
+
+      ls = &lsr[language_index].LangSys;
+    }
+    fi = ls->FeatureIndex;
+
+    if ( ALLOC_ARRAY( ftl, ls->FeatureCount + 1, TT_ULong ) )
+      return error;
+
+    for ( n = 0; n < ls->FeatureCount; n++ )
+    {
+      if ( fi[n] >= fl->FeatureCount )
+      {
+        FREE( ftl );
+        return TTO_Err_Invalid_GSUB_SubTable_Format;
+      }
+      ftl[n] = fr[fi[n]].FeatureTag;
+    }
+    ftl[n] = 0;
+
+    *feature_tag_list = ftl;
+
+    return TT_Err_Ok;
+  }
+
+
 /* END */
