@@ -37,7 +37,9 @@
   {
     DEFINE_LOAD_LOCALS( input->stream );
 
-    UShort  n, count;
+    UShort   n, count;
+
+    UShort*  fi;
 
 
     if ( ACCESS_Frame( 6L ) )
@@ -57,8 +59,10 @@
     if ( ACCESS_Frame( count * 2L ) )
       return error;
 
+    fi = ls->FeatureIndex;
+
     for ( n = 0; n < count; n++ )
-      ls->FeatureIndex[n] = GET_UShort();
+      fi[n] = GET_UShort();
 
     FORGET_Frame();
 
@@ -80,8 +84,10 @@
   {
     DEFINE_LOAD_LOCALS( input->stream );
 
-    UShort  n, count;
-    ULong   cur_offset, new_offset, base_offset;
+    UShort              n, count;
+    ULong               cur_offset, new_offset, base_offset;
+
+    TTO_LangSysRecord*  lsr;
 
 
     base_offset = FILE_Pos();
@@ -124,20 +130,21 @@
     if ( ALLOC_ARRAY( s->LangSysRecord, count, TTO_LangSysRecord ) )
       return error;
 
+    lsr = s->LangSysRecord;
+
     for ( n = 0; n < count; n++ )
     {
       if ( ACCESS_Frame( 6L ) )
         return error;
 
-      s->LangSysRecord[n].LangSysTag = GET_ULong();
+      lsr[n].LangSysTag = GET_ULong();
       new_offset = GET_UShort() + base_offset;
 
       FORGET_Frame();
 
       cur_offset = FILE_Pos();
       if ( FILE_Seek( new_offset ) ||
-           ( error = Load_LangSys( &s->LangSysRecord[n].LangSys,
-                                   input ) ) != TT_Err_Ok )
+           ( error = Load_LangSys( &lsr[n].LangSys, input ) ) != TT_Err_Ok )
         return error;
       (void)FILE_Seek( cur_offset );
     }
@@ -172,8 +179,10 @@
   {
     DEFINE_LOAD_LOCALS( input->stream );
 
-    UShort  n, count;
-    ULong   cur_offset, new_offset, base_offset;
+    UShort             n, count;
+    ULong              cur_offset, new_offset, base_offset;
+
+    TTO_ScriptRecord*  sr;
 
 
     if ( ACCESS_Frame( 2L ) )
@@ -198,20 +207,21 @@
     if ( ALLOC_ARRAY( sl->ScriptRecord, count, TTO_ScriptRecord ) )
       return error;
 
+    sr = sl->ScriptRecord;
+
     for ( n = 0; n < count; n++ )
     {
       if ( ACCESS_Frame( 6L ) )
         return error;
 
-      sl->ScriptRecord[n].ScriptTag = GET_ULong();
+      sr[n].ScriptTag = GET_ULong();
       new_offset = GET_UShort() + base_offset;
 
       FORGET_Frame();
 
       cur_offset = FILE_Pos();
       if ( FILE_Seek( new_offset ) ||
-           ( error = Load_Script( &sl->ScriptRecord[n].Script,
-                                  input ) ) != TT_Err_Ok )
+           ( error = Load_Script( &sr[n].Script, input ) ) != TT_Err_Ok )
         return error;
       (void)FILE_Seek( cur_offset );
     }
@@ -250,7 +260,9 @@
   {
     DEFINE_LOAD_LOCALS( input->stream );
 
-    UShort  n, count;
+    UShort   n, count;
+
+    UShort*  lli;
 
 
     if ( ACCESS_Frame( 4L ) )
@@ -266,11 +278,13 @@
     if ( ALLOC_ARRAY( f->LookupListIndex, count, TT_UShort ) )
       return error;
 
+    lli = f->LookupListIndex;
+
     if ( ACCESS_Frame( count * 2L ) )
       return error;
 
     for ( n = 0; n < count; n++ )
-      f->LookupListIndex[n] = GET_UShort();
+      lli[n] = GET_UShort();
 
     FORGET_Frame();
 
@@ -292,8 +306,10 @@
   {
     DEFINE_LOAD_LOCALS( input->stream );
 
-    UShort  n, count;
-    ULong   cur_offset, new_offset, base_offset;
+    UShort              n, count;
+    ULong               cur_offset, new_offset, base_offset;
+
+    TTO_FeatureRecord*  fr;
 
 
     if ( ACCESS_Frame( 2L ) )
@@ -318,20 +334,21 @@
     if ( ALLOC_ARRAY( fl->FeatureRecord, count, TTO_FeatureRecord ) )
       return error;
 
+    fr = fl->FeatureRecord;
+
     for ( n = 0; n < count; n++ )
     {
       if ( ACCESS_Frame( 6L ) )
         return error;
 
-      fl->FeatureRecord[n].FeatureTag = GET_ULong();
+      fr[n].FeatureTag = GET_ULong();
       new_offset = GET_UShort() + base_offset;
 
       FORGET_Frame();
 
       cur_offset = FILE_Pos();
       if ( FILE_Seek( new_offset ) ||
-           ( error = Load_Feature( &fl->FeatureRecord[n].Feature,
-                                   input ) ) != TT_Err_Ok )
+           ( error = Load_Feature( &fr[n].Feature, input ) ) != TT_Err_Ok )
         return error;
       (void)FILE_Seek( cur_offset );
     }
@@ -370,10 +387,10 @@
 
   static TT_Error  Load_SubTable( TTO_SubTable*  st,
                                   PFace          input,
-                                  TTO_LoadType   table_type,
+                                  TTO_Type       table_type,
                                   UShort         lookup_type )
   {
-    if ( table_type == Load_GSUB )
+    if ( table_type == GSUB )
       switch ( lookup_type )
       {
       case LOOKUP_SINGLE:
@@ -397,10 +414,10 @@
 
 
   static void  Free_SubTable( TTO_SubTable*  st,
-                              TTO_LoadType   table_type,
+                              TTO_Type       table_type,
                               UShort         lookup_type )
   {
-    if ( table_type == Load_GSUB )
+    if ( table_type == GSUB )
       switch ( lookup_type )
       {
       case LOOKUP_SINGLE:
@@ -430,12 +447,14 @@
 
   static TT_Error  Load_Lookup( TTO_Lookup*   l,
                                 PFace         input,
-                                TTO_LoadType  type )
+                                TTO_Type      type )
   {
     DEFINE_LOAD_LOCALS( input->stream );
 
-    UShort  n, count;
-    ULong   cur_offset, new_offset, base_offset;
+    UShort         n, count;
+    ULong          cur_offset, new_offset, base_offset;
+
+    TTO_SubTable*  st;
 
 
     base_offset = FILE_Pos();
@@ -454,6 +473,8 @@
     if ( ALLOC_ARRAY( l->SubTable, count, TTO_SubTable ) )
       return error;
 
+    st = l->SubTable;
+
     for ( n = 0; n < count; n++ )
     {
       if ( ACCESS_Frame( 2L ) )
@@ -465,7 +486,7 @@
 
       cur_offset = FILE_Pos();
       if ( FILE_Seek( new_offset ) ||
-           ( error = Load_SubTable( &l->SubTable[n], input,
+           ( error = Load_SubTable( &st[n], input,
                                     type, l->LookupType ) ) != TT_Err_Ok )
         return error;
       (void)FILE_Seek( cur_offset );
@@ -476,7 +497,7 @@
 
 
   static void  Free_Lookup( TTO_Lookup*   l,
-                            TTO_LoadType  type )
+                            TTO_Type      type )
   {
     UShort  n, count;
 
@@ -497,12 +518,14 @@
 
   TT_Error  Load_LookupList( TTO_LookupList*  ll,
                              PFace            input,
-                             TTO_LoadType     type )
+                             TTO_Type         type )
   {
     DEFINE_LOAD_LOCALS( input->stream );
 
-    UShort  n, count;
-    ULong   cur_offset, new_offset, base_offset;
+    UShort       n, count;
+    ULong        cur_offset, new_offset, base_offset;
+
+    TTO_Lookup*  l;
 
 
     if ( ACCESS_Frame( 2L ) )
@@ -527,6 +550,8 @@
     if ( ALLOC_ARRAY( ll->Lookup, count, TTO_Lookup ) )
       return error;
 
+    l = ll->Lookup;
+
     for ( n = 0; n < count; n++ )
     {
       if ( ACCESS_Frame( 2L ) )
@@ -538,8 +563,7 @@
 
       cur_offset = FILE_Pos();
       if ( FILE_Seek( new_offset ) ||
-           ( error = Load_Lookup( &ll->Lookup[n],
-                                  input, type ) ) != TT_Err_Ok )
+           ( error = Load_Lookup( &l[n], input, type ) ) != TT_Err_Ok )
         return error;
       (void)FILE_Seek( cur_offset );
     }
@@ -549,7 +573,7 @@
 
 
   void  Free_LookupList( TTO_LookupList*  ll,
-                         TTO_LoadType     type )
+                         TTO_Type         type )
   {
     UShort  n, count;
 
@@ -579,7 +603,9 @@
   {
     DEFINE_LOAD_LOCALS( input->stream );
 
-    UShort  n, count;
+    UShort   n, count;
+
+    UShort*  ga;
 
 
     if ( ACCESS_Frame( 2L ) )
@@ -594,11 +620,13 @@
     if ( ALLOC_ARRAY( cf1->GlyphArray, count, TT_UShort ) )
       return error;
 
+    ga = cf1->GlyphArray;
+
     if ( ACCESS_Frame( count * 2L ) )
       return error;
 
     for ( n = 0; n < count; n++ )
-      cf1->GlyphArray[n] = GET_UShort();
+      ga[n] = GET_UShort();
 
     FORGET_Frame();
 
@@ -620,7 +648,9 @@
   {
     DEFINE_LOAD_LOCALS( input->stream );
 
-    UShort  n, count;
+    UShort            n, count;
+
+    TTO_RangeRecord*  rr;
 
 
     if ( ACCESS_Frame( 2L ) )
@@ -635,14 +665,22 @@
     if ( ALLOC_ARRAY( cf2->RangeRecord, count, TTO_RangeRecord ) )
       return error;
 
+    rr = cf2->RangeRecord;
+
     if ( ACCESS_Frame( count * 6L ) )
       return error;
 
     for ( n = 0; n < count; n++ )
     {
-      cf2->RangeRecord[n].Start              = GET_UShort();
-      cf2->RangeRecord[n].End                = GET_UShort();
-      cf2->RangeRecord[n].StartCoverageIndex = GET_UShort();
+      rr[n].Start              = GET_UShort();
+      rr[n].End                = GET_UShort();
+      rr[n].StartCoverageIndex = GET_UShort();
+
+      /* sanity check; we are working with 16bit integers */
+      if ( rr[n].Start > rr[n].End ||
+           ( rr[n].End - rr[n].Start + rr[n].StartCoverageIndex ) <
+             rr[n].StartCoverageIndex )
+        return TTO_Err_Invalid_SubTable;
     }
 
     FORGET_Frame();
@@ -701,6 +739,118 @@
   }
 
 
+  static TT_Error  Coverage_Index1( TTO_CoverageFormat1*  cf1,
+                                    UShort                glyphID,
+                                    UShort*               index )
+  {
+    UShort   min, max, new_min, new_max, middle;
+
+    UShort*  array = cf1->GlyphArray;
+
+
+    /* binary search */
+
+    new_min = 0;
+    new_max = cf1->GlyphCount - 1;
+
+    do
+    {
+      min = new_min;
+      max = new_max;
+
+      /* we use (min + max) / 2 = max - (max - min) / 2  to avoid
+         overflow and rounding errors                             */
+
+      middle = max - ( ( max - min ) >> 1 );
+
+      if ( glyphID == array[middle] )
+      {
+        *index = middle;
+        return TT_Err_Ok;
+      }
+      else if ( glyphID < array[middle] )
+      {
+        if ( middle == min )
+          break;
+        new_max = middle - 1;
+      }
+      else
+      {
+        if ( middle == max )
+          break;
+        new_min = middle + 1;
+      }
+    } while ( min < max );
+
+    return TTO_Err_Not_Covered;
+  }
+
+
+  static TT_Error  Coverage_Index2( TTO_CoverageFormat2*  cf2,
+                                    UShort                glyphID,
+                                    UShort*               index )
+  {
+    UShort            min, max, new_min, new_max, middle;
+
+    TTO_RangeRecord*  rr = cf2->RangeRecord;
+
+
+    /* binary search */
+
+    new_min = 0;
+    new_max = cf2->RangeCount - 1;
+
+    do
+    {
+      min = new_min;
+      max = new_max;
+
+      /* we use (min + max) / 2 = max - (max - min) / 2  to avoid
+         overflow and rounding errors                             */
+
+      middle = max - ( ( max - min ) >> 1 );
+
+      if ( glyphID >= rr[middle].Start && glyphID <= rr[middle].End )
+      {
+        *index = rr[middle].StartCoverageIndex + glyphID - rr[middle].Start;
+        return TT_Err_Ok;
+      }
+      else if ( glyphID < rr[middle].Start )
+      {
+        if ( middle == min )
+          break;
+        new_max = middle - 1;
+      }
+      else
+      {
+        if ( middle == max )
+          break;
+        new_min = middle + 1;
+      }
+    } while ( min < max );
+
+    return TTO_Err_Not_Covered;
+  }
+
+
+  TT_Error  Coverage_Index( TTO_Coverage*  c,
+                            UShort         glyphID,
+                            UShort*        index )
+  {
+    switch ( c->CoverageFormat )
+    {
+    case 1:
+      return Coverage_Index1( &c->cf.cf1, glyphID, index );
+    case 2:
+      return Coverage_Index2( &c->cf.cf2, glyphID, index );
+    default:
+      return TTO_Err_Invalid_SubTable_Format;
+    }
+
+    return TT_Err_Ok;               /* never reached */
+  }
+
+
 
   /*************************************
    * Class Definition related functions
@@ -725,6 +875,11 @@
 
     FORGET_Frame();
 
+    /* sanity check; we are working with 16bit integers */
+
+    if ( cdf1->StartGlyph + count < cdf1->StartGlyph )
+      return TTO_Err_Invalid_SubTable;
+
     cdf1->ClassValueArray = NULL;
 
     if ( ALLOC_ARRAY( cdf1->ClassValueArray, count, TT_UShort ) )
@@ -756,7 +911,8 @@
   {
     DEFINE_LOAD_LOCALS( input->stream );
 
-    UShort  n, count;
+    UShort                 n, count;
+    TTO_ClassRangeRecord*  crr;
 
 
     if ( ACCESS_Frame( 2L ) )
@@ -771,14 +927,20 @@
     if ( ALLOC_ARRAY( cdf2->ClassRangeRecord, count, TTO_ClassRangeRecord ) )
       return error;
 
+    crr = cdf2->ClassRangeRecord;
+
     if ( ACCESS_Frame( count * 6L ) )
       return error;
 
     for ( n = 0; n < count; n++ )
     {
-      cdf2->ClassRangeRecord[n].Start = GET_UShort();
-      cdf2->ClassRangeRecord[n].End   = GET_UShort();
-      cdf2->ClassRangeRecord[n].Class = GET_UShort();
+      crr[n].Start = GET_UShort();
+      crr[n].End   = GET_UShort();
+      crr[n].Class = GET_UShort();
+
+      /* sanity check */
+      if ( crr[n].Start > crr[n].End )
+        return TTO_Err_Invalid_SubTable;
     }
 
     FORGET_Frame();
@@ -839,4 +1001,78 @@
   }
 
 
+  static UShort  Get_Class1( TTO_ClassDefFormat1*  cdf1,
+                             UShort                glyphID )
+  {
+    UShort*  cva = cdf1->ClassValueArray;
+
+
+    if ( glyphID >= cdf1->StartGlyph &&
+         glyphID <= cdf1->StartGlyph + cdf1->GlyphCount )
+      return cva[glyphID - cdf1->StartGlyph];
+
+    return 0;
+  }
+
+
+  static UShort  Get_Class2( TTO_ClassDefFormat2*  cdf2,
+                             UShort                glyphID )
+  {
+    UShort                 min, max, new_min, new_max, middle;
+
+    TTO_ClassRangeRecord*  crr = cdf2->ClassRangeRecord;
+
+
+    /* binary search */
+
+    new_min = 0;
+    new_max = cdf2->ClassRangeCount - 1;
+
+    do
+    {
+      min = new_min;
+      max = new_max;
+
+      /* we use (min + max) / 2 = max - (max - min) / 2  to avoid
+         overflow and rounding errors                             */
+
+      middle = max - ( ( max - min ) >> 1 );
+
+      if ( glyphID >= crr[middle].Start && glyphID <= crr[middle].End )
+        return crr[middle].Class;
+      else if ( glyphID < crr[middle].Start )
+      {
+        if ( middle == min )
+          break;
+        new_max = middle - 1;
+      }
+      else
+      {
+        if ( middle == max )
+          break;
+        new_min = middle + 1;
+      }
+    } while ( min < max );
+
+    return 0;
+  }
+
+
+  UShort  Get_Class( TTO_ClassDefinition*  cd,
+                     UShort                glyphID )
+  {
+    switch ( cd->ClassFormat )
+    {
+    case 1:
+      return Get_Class1( &cd->cd.cd1, glyphID );
+    case 2:
+      return Get_Class2( &cd->cd.cd2, glyphID );
+    default:
+      break;
+    }
+
+    return 0;
+  }
+
+
 /* END */
