@@ -1,6 +1,6 @@
 /*******************************************************************
  *
- *  ttengine.h                                                   1.0
+ *  ttengine.h                                                   1.1
  *
  *    Engine instance structure definition.
  *
@@ -13,11 +13,18 @@
  *  this file you indicate that you have read the license and
  *  understand and accept it fully.
  *
+ *  New in 1.1 :
+ *
+ *    - added the 'raster_lock' mutex field to synchronize
+ *      scan-line conversion in thread-safe and re-entrant builds.
+ *
  ******************************************************************/
 
 #ifndef TTENGINE_H
 #define TTENGINE_H
 
+#include "tttypes.h"
+#include "ttconfig.h"
 #include "freetype.h"
 #include "ttmutex.h"
 
@@ -40,7 +47,7 @@
   /*                                                                  */
   /********************************************************************/
 
-  struct  _TEngine_Instance 
+  struct  TEngine_Instance_
   {
      TMutex  lock;               /* engine lock */
 
@@ -52,22 +59,33 @@
      void*   objs_glyph_class;      /* the glyph cache class    */
 
      void*   objs_face_cache;  /* these caches are used to track */
+     void*   objs_exec_cache;  /* the current face and execution */
+                               /* context objects                */
 
      void*   file_component;    /* ttfile implementation dependent   */
 
-     void*   raster_component;  /* ttraster implementation depedent  */
-     char    raster_palette[5]; /* gray-levels palette for anti-aliasing */
+     TMutex  raster_lock;       /* mutex for this engine's render pool   */
+     void*   raster_component;  /* ttraster implementation depedent      */
+     Byte    raster_palette[5]; /* gray-levels palette for anti-aliasing */
 
      void*   extension_component;  /* extensions dependent */
 
+#if 0
      TT_Glyph_Loader_Callback  glCallback; /* glyph loader callback, if any */
+#endif
   };
 
-  typedef struct _TEngine_Instance  TEngine_Instance;
+  /* NOTE : The raster's lock is only acquired by the Render_Glyph and     */
+  /*        Render_Gray_Glyph functions, which always release it on exit   */
+  /*        They do not lock the engine mutex. This means you shouldn't    */
+  /*        be concerned about deadlocks between the two mutexes, as these */
+  /*        should never appear..                                          */
+
+  typedef struct TEngine_Instance_  TEngine_Instance;
   typedef TEngine_Instance*         PEngine_Instance;
 
 
-#ifdef TT_CONFIG_REENTRANT  /* for re-entrant builds */
+#ifdef TT_CONFIG_OPTION_THREAD_SAFE  /* for re-entrant builds */
 
 #define ENGINE_ARG    TEngine_Instance*  _engine
 #define ENGINE_ARGS   TEngine_Instance*  _engine,
@@ -85,7 +103,7 @@
 #define ENGINE_VAR
 #define ENGINE_VARS
 
-#endif /* TT_CONFIG_REENTRANT */
+#endif /* TT_CONFIG_OPTION_THREAD_SAFE */
 
 #ifdef __cplusplus
   }
