@@ -9,7 +9,7 @@
  *
  *  This file is part of the FreeType project, and may only be used
  *  modified and distributed under the terms of the FreeType project
- *  license, LICENSE.TXT.  By continuing to use, modify, or distribute 
+ *  license, LICENSE.TXT.  By continuing to use, modify, or distribute
  *  this file you indicate that you have read the license and
  *  understand and accept it fully.
  *
@@ -33,18 +33,21 @@
 #ifndef TTFILE_H
 #define TTFILE_H
 
+#include "ttconfig.h"
 #include "freetype.h"
 #include "ttengine.h"
-#include "tterror.h"
+#include "ttdebug.h"
 
 #ifdef __cplusplus
   extern "C" {
 #endif
 
-  /* Initialize file component */  
+  /* Initialize file component */
+  LOCAL_DEF
   TT_Error  TTFile_Init( PEngine_Instance  engine );
 
   /* Done with file component */
+  LOCAL_DEF
   TT_Error  TTFile_Done( PEngine_Instance  engine );
 
 
@@ -57,13 +60,15 @@
   /* Open a file and return a stream handle for it.           */
   /* Should only be used for a new face object's main stream. */
 
-  TT_Error  TT_Open_Stream( const char*  name,
-                                   TT_Stream*   stream );
+  LOCAL_DEF
+  TT_Error  TT_Open_Stream( const TT_Text* name,
+                            TT_Stream*     stream );
 
 
   /* Closes, then discards, a stream when it's no longer needed.   */
   /* Should only be used for a stream opend with TT_Open_Stream(). */
 
+  LOCAL_DEF
   TT_Error  TT_Close_Stream( TT_Stream*  stream );
 
 
@@ -75,6 +80,7 @@
   /* to 'stream'. Otherwise, the latter is a duplicate handle  */
   /* for the file opened with 'org_stream'                     */
 
+  EXPORT_DEF
   TT_Error  TT_Use_Stream( TT_Stream   org_stream,
                            TT_Stream*  stream );
 
@@ -84,19 +90,21 @@
 
   /* in re-entrant builds, this will really discard the stream */
 
+  EXPORT_DEF
   TT_Error  TT_Done_Stream( TT_Stream*  stream );
 
   /* Closes the stream's file handle to release system resources */
   /* The function TT_Use_Stream automatically re-activates a     */
   /* flushed stream when it uses one                             */
 
+  EXPORT_DEF
   TT_Error  TT_Flush_Stream( TT_Stream*  stream );
 
 /* The macros STREAM_ARGS and STREAM_ARG let us build a thread-safe */
 /* or re-entrant implementation depending on a single configuration */
 /*define.                                                           */
 
-#ifdef TT_CONFIG_REENTRANT
+#ifdef TT_CONFIG_OPTION_THREAD_SAFE
 
 #define STREAM_ARGS   TT_Stream  stream,
 #define STREAM_ARG    TT_Stream  stream
@@ -104,7 +112,7 @@
 #else
 
 #define STREAM_ARGS   /* void */
-#define STREAM_ARG
+#define STREAM_ARG    void
 
 #endif /* TT_CONFIG_REENTRANT */
 
@@ -122,33 +130,38 @@
 
   /* Read 'count' bytes from file into 'buffer' */
 
-  TT_Error  TT_Read_File( STREAM_ARGS void*  buffer, 
-                                      long   count );
+  EXPORT_DEF
+  TT_Error  TT_Read_File( STREAM_ARGS void*   buffer,
+                                      Long    count );
 
 
   /* Seek file cursor to a given position */
 
-  TT_Error  TT_Seek_File( STREAM_ARGS long  position );
+  EXPORT_DEF
+  TT_Error  TT_Seek_File( STREAM_ARGS Long  position );
 
 
   /* Skip the next 'distance' bytes in file */
 
-  TT_Error  TT_Skip_File( STREAM_ARGS long  distance );
+  EXPORT_DEF
+  TT_Error  TT_Skip_File( STREAM_ARGS Long  distance );
 
 
   /* Read the 'count' bytes at 'position' into 'buffer' */
 
-  TT_Error  TT_Read_At_File( STREAM_ARGS long   position,
+  EXPORT_DEF
+  TT_Error  TT_Read_At_File( STREAM_ARGS Long   position,
                                          void*  buffer,
-                                         long   count );
-
+                                         Long   count );
 
   /* Return current file position */
 
+  EXPORT_DEF
   Long  TT_File_Pos( STREAM_ARG );
 
   /* Return length of a given stream, even if it is flushed */
 
+  EXPORT_DEF
   Long  TT_Stream_Size( TT_Stream  stream );
 
 
@@ -163,16 +176,17 @@
 
   /* Frame type declaration.*/
 
-  struct  _TFileFrame
+  struct  TFileFrame_
   {
-    unsigned char*  address;  /* frame buffer                     */
-    unsigned char*  cursor;   /* current cursor position in frame */
-    int             size;     /* frame size                       */
+    Byte*  address;  /* frame buffer                     */
+    Byte*  cursor;   /* current cursor position in frame */
+    Long   size;     /* frame size                       */
   };
 
-  typedef struct _TFileFrame  TFileFrame;
+  typedef struct TFileFrame_  TFileFrame;
 
-  extern const TFileFrame  TT_Null_FileFrame;
+  EXPORT_DEF
+  const TFileFrame  TT_Null_FileFrame;
 
 
 /* The macro ZERO_Frame is used to define and init a frame.      */
@@ -192,14 +206,15 @@
 /* or re-entrant implementation depending on a single configuration */
 /* define                                                           */
 
-#ifdef TT_CONFIG_REENTRANT
+#ifdef TT_CONFIG_OPTION_THREAD_SAFE
 
-  #define FRAME_ARGS   TFileFrame* frame,
-  #define FRAME_ARG    TFileFrame* frame
+#define FRAME_ARGS   TFileFrame* frame,
+#define FRAME_ARG    TFileFrame* frame
 
 #else
-  #define FRAME_ARGS  /* void */
-  #define FRAME_ARG   /* void */
+
+#define FRAME_ARGS   /* void */
+#define FRAME_ARG    void
 
 #endif /* TT_CONFIG_REENTRANT */
 
@@ -207,29 +222,42 @@
   /* Access the next 'size' bytes from current position. */
   /* Fails if all bytes cannot be read/accessed.         */
 
-  TT_Error  TT_Access_Frame( STREAM_ARGS FRAME_ARGS int  size );
+  EXPORT_DEF
+  TT_Error  TT_Access_Frame( STREAM_ARGS FRAME_ARGS Long  size );
 
 
   /* Access the bytes located in the next 'size' bytes of the file. */
   /* Doesn't fail if less than 'size' bytes are accessible (like    */
   /* at the end of the file).                                       */
 
-  TT_Error  TT_Check_And_Access_Frame( STREAM_ARGS FRAME_ARGS int  size );
-
+  EXPORT_DEF
+  TT_Error  TT_Check_And_Access_Frame( STREAM_ARGS FRAME_ARGS Long  size );
 
   /* Forget frame */
 
+  EXPORT_DEF
   TT_Error  TT_Forget_Frame( FRAME_ARG );
 
 
   /* primitive routines for data accessing */
 
-  unsigned char   TT_Get_Byte  ( FRAME_ARG );
-  char            TT_Get_Char  ( FRAME_ARG );
-  short           TT_Get_Short ( FRAME_ARG );
-  unsigned short  TT_Get_UShort( FRAME_ARG );
-  long            TT_Get_Long  ( FRAME_ARG );
-  unsigned long   TT_Get_ULong ( FRAME_ARG );
+  EXPORT_DEF Char   TT_Get_Char  ( FRAME_ARG );
+  EXPORT_DEF Short  TT_Get_Short ( FRAME_ARG );
+  EXPORT_DEF Long   TT_Get_Long  ( FRAME_ARG );
+
+#ifdef TT_CONFIG_OPTION_THREAD_SAFE
+
+#define  TT_Get_Byte( frame )   ( (Byte  )TT_Get_Char ( frame ) )
+#define  TT_Get_UShort( frame ) ( (UShort)TT_Get_Short( frame ) )
+#define  TT_Get_ULong( frame )  ( (ULong )TT_Get_Long ( frame ) )
+
+#else
+
+#define  TT_Get_Byte()   ((Byte  )TT_Get_Char ())
+#define  TT_Get_UShort() ((UShort)TT_Get_Short())
+#define  TT_Get_ULong()  ((ULong )TT_Get_Long ())
+
+#endif /* TT_CONFIG_REENTRANT */
 
 
 #ifdef __cplusplus
