@@ -2,7 +2,7 @@
  *
  *  ttload.h                                                    1.1
  *
- *    TrueType Tables Loader.                          
+ *    TrueType Tables Loader.
  *
  *  Copyright 1996-1998 by
  *  David Turner, Robert Wilhelm, and Werner Lemberg.
@@ -23,42 +23,43 @@
 #ifndef TTLOAD_H
 #define TTLOAD_H
 
-#include "ttcommon.h"
-
+#include "ttconfig.h"
+#include "tttypes.h"
+#include "ttobjs.h"
 
 #ifdef __cplusplus
   extern "C" {
 #endif
 
-  Int  LookUp_TrueType_Table( PFace  face,
-                              Long   tag  );
+  EXPORT_DEF Long  TT_LookUp_Table( PFace  face,
+                                    ULong  tag  );
 
-  TT_Error  Load_TrueType_Directory        ( PFace  face, 
-                                             int    faceIndex );
+  LOCAL_DEF TT_Error  Load_TrueType_Directory        ( PFace  face,
+                                                       ULong  faceIndex );
 
-  TT_Error  Load_TrueType_MaxProfile       ( PFace  face );
-  TT_Error  Load_TrueType_Gasp             ( PFace  face );
-  TT_Error  Load_TrueType_Header           ( PFace  face );
-  TT_Error  Load_TrueType_Horizontal_Header( PFace  face );
-  TT_Error  Load_TrueType_Locations        ( PFace  face );
-  TT_Error  Load_TrueType_Names            ( PFace  face );
-  TT_Error  Load_TrueType_CVT              ( PFace  face );
-  TT_Error  Load_TrueType_CMap             ( PFace  face );
-  TT_Error  Load_TrueType_HMTX             ( PFace  face );
-  TT_Error  Load_TrueType_Programs         ( PFace  face );
-  TT_Error  Load_TrueType_OS2              ( PFace  face );
-  TT_Error  Load_TrueType_PostScript       ( PFace  face );
-  TT_Error  Load_TrueType_Hdmx             ( PFace  face );
+  LOCAL_DEF TT_Error  Load_TrueType_MaxProfile       ( PFace  face );
+  LOCAL_DEF TT_Error  Load_TrueType_Gasp             ( PFace  face );
+  LOCAL_DEF TT_Error  Load_TrueType_Header           ( PFace  face );
+  LOCAL_DEF TT_Error  Load_TrueType_Locations        ( PFace  face );
+  LOCAL_DEF TT_Error  Load_TrueType_Names            ( PFace  face );
+  LOCAL_DEF TT_Error  Load_TrueType_CVT              ( PFace  face );
+  LOCAL_DEF TT_Error  Load_TrueType_CMap             ( PFace  face );
+  LOCAL_DEF TT_Error  Load_TrueType_Programs         ( PFace  face );
+  LOCAL_DEF TT_Error  Load_TrueType_OS2              ( PFace  face );
+  LOCAL_DEF TT_Error  Load_TrueType_PostScript       ( PFace  face );
+  LOCAL_DEF TT_Error  Load_TrueType_Hdmx             ( PFace  face );
 
-  TT_Error  Load_TrueType_Any( PFace  face,
-                               Long   tag,
-                               Long   offset,
-                               void*  buffer,
-                               Long*  length );
+  LOCAL_DEF TT_Error  Load_TrueType_Metrics_Header( PFace  face,
+                                                    Bool   vertical );
 
+  LOCAL_DEF TT_Error  Load_TrueType_Any( PFace  face,
+                                         ULong  tag,
+                                         Long   offset,
+                                         void*  buffer,
+                                         Long*  length );
 
-  TT_Error  Free_TrueType_Names( PFace  face );
-  TT_Error  Free_TrueType_Hdmx ( PFace  face );
+  LOCAL_DEF TT_Error  Free_TrueType_Names( PFace  face );
+  LOCAL_DEF TT_Error  Free_TrueType_Hdmx ( PFace  face );
 
 
 /* The following macros are defined to simplify the writing of */
@@ -67,7 +68,7 @@
 /* For examples see the code in ttload.c, ttgload.c etc.       */
 
 #define USE_Stream( original, duplicate ) \
-          ( error = TT_Use_Stream( original, &duplicate ) )
+          ( (error = TT_Use_Stream( original, &duplicate )) != TT_Err_Ok )
 
 #define DONE_Stream( _stream ) \
           TT_Done_Stream( &_stream )
@@ -79,7 +80,7 @@
 #define DEFINE_A_STREAM  TT_Stream   stream
 
 
-#ifdef TT_CONFIG_REENTRANT  /* re-entrant implementation */
+#ifdef TT_CONFIG_OPTION_THREAD_SAFE  /* re-entrant implementation */
 
 /* The following macros define the necessary local */
 /* variables used to access streams and frames.    */
@@ -114,31 +115,31 @@
 
 
 #define ACCESS_Frame( _size_ ) \
-          ( error = TT_Access_Frame( stream, &frame, _size_ ) )
+          ( (error = TT_Access_Frame( stream, &frame, _size_ )) != TT_Err_Ok )
 #define CHECK_ACCESS_Frame( _size_ ) \
-          ( error = TT_Check_And_Access_Frame( stream, &frame, _size_ ) )
+          ( (error = TT_Check_And_Access_Frame( stream, &frame, _size_ )) != TT_Err_Ok )
 #define FORGET_Frame() \
-          ( error = TT_Forget_Frame( &frame ) )
-  
+          ( (void)TT_Forget_Frame( &frame ) )
+
 #define GET_Byte()    TT_Get_Byte  ( &frame )
 #define GET_Char()    TT_Get_Char  ( &frame )
 #define GET_UShort()  TT_Get_UShort( &frame )
 #define GET_Short()   TT_Get_Short ( &frame )
 #define GET_Long()    TT_Get_Long  ( &frame )
 #define GET_ULong()   TT_Get_ULong ( &frame )
-#define GET_Tag4()    TT_Get_Long  ( &frame )
-  
+#define GET_Tag4()    TT_Get_ULong ( &frame )
+
 #define FILE_Pos()    TT_File_Pos ( stream )
 
 #define FILE_Seek( _position_ ) \
-          ( error = TT_Seek_File( stream, _position_ ) )
+          ( (error = TT_Seek_File( stream, _position_ )) != TT_Err_Ok )
 #define FILE_Skip( _distance_ ) \
-          ( error = TT_Skip_File( stream, _distance_ ) )
+          ( (error = TT_Skip_File( stream, _distance_ )) != TT_Err_Ok )
 #define FILE_Read( buffer, count ) \
-          ( error = TT_Read_File ( stream, buffer, count ) )
+          ( (error = TT_Read_File ( stream, buffer, count )) != TT_Err_Ok )
 #define FILE_Read_At( pos, buffer, count ) \
-          ( error = TT_Read_At_File( stream, pos, buffer, count ) )
-  
+          ( (error = TT_Read_At_File( stream, pos, buffer, count )) != TT_Err_Ok )
+
 #else   /* thread-safe implementation */
 
 /* Define stream locals with frame -- nothing in thread-safe mode */
@@ -153,6 +154,7 @@
 #define DEFINE_LOAD_LOCALS( STREAM ) \
           TT_Error  error
 
+
 /* Define locals without frame with a predefined stream - see ttload.c */
 #define DEFINE_LOAD_LOCALS_WO_FRAME( STREAM ) \
           TT_Error  error
@@ -164,32 +166,32 @@
 
 
 #define ACCESS_Frame( _size_ ) \
-          ( error = TT_Access_Frame( _size_ ) )
+          ( (error = TT_Access_Frame( _size_ )) != TT_Err_Ok )
 #define CHECK_ACCESS_Frame( _size_ ) \
-          ( error = TT_Check_And_Access_Frame( _size_ ) )
+          ( (error = TT_Check_And_Access_Frame( _size_ )) != TT_Err_Ok )
 #define FORGET_Frame() \
-          ( error = TT_Forget_Frame() )
-  
+          ( (void)TT_Forget_Frame() )
+
 #define GET_Byte()    TT_Get_Byte  ()
 #define GET_Char()    TT_Get_Char  ()
 #define GET_UShort()  TT_Get_UShort()
 #define GET_Short()   TT_Get_Short ()
 #define GET_Long()    TT_Get_Long  ()
 #define GET_ULong()   TT_Get_ULong ()
-#define GET_Tag4()    TT_Get_Long  ()
-  
+#define GET_Tag4()    TT_Get_ULong ()
+
 #define FILE_Pos()    TT_File_Pos()
 
 #define FILE_Seek( _position_ ) \
-          ( error = TT_Seek_File( _position_ ) )
+          ( (error = TT_Seek_File( _position_ )) != TT_Err_Ok )
 #define FILE_Skip( _distance_ ) \
-          ( error = TT_Skip_File( _distance_ ) )
+          ( (error = TT_Skip_File( _distance_ )) != TT_Err_Ok )
 #define FILE_Read( buffer, count ) \
-          ( error = TT_Read_File ( buffer, count ) )
+          ( (error = TT_Read_File ( buffer, count )) != TT_Err_Ok )
 #define FILE_Read_At( pos, buffer, count ) \
-          ( error = TT_Read_At_File( pos, buffer, count ) )
-  
-#endif /* TT_CONFIG_REENTRANT */
+          ( (error = TT_Read_At_File( pos, buffer, count )) != TT_Err_Ok )
+
+#endif /* TT_CONFIG_OPTION_THREAD_SAFE */
 
 #ifdef __cplusplus
   }
