@@ -53,7 +53,12 @@
 
   static Int  num_alloc;
   static Int  num_free;
+  static Int  num_realloc; /* counts only `real' reallocations
+                              (i.e., an existing buffer will be resized
+                              to a value larger than zero */
+
   static Int  fail_alloc;
+  static Int  fail_realloc;
   static Int  fail_free;
 
 #endif /* DEBUG_MEMORY */
@@ -76,9 +81,9 @@
  *
  *  Output :  Error code.
  *
- *  NOTE:  The newly allocated block should _always_ be zeroed
- *         on return.  Many parts of the engine rely on this to
- *         work properly.
+ *  NOTE :  The newly allocated block should _always_ be zeroed
+ *          on return.  Many parts of the engine rely on this to
+ *          work properly.
  *
  ******************************************************************/
 
@@ -90,9 +95,12 @@
 #endif
 
 
-    if ( Size )
+    if ( !P )
+      return TT_Err_Invalid_Argument;
+
+    if ( Size > 0 )
     {
-      *P = (void *) malloc( Size );
+      *P = (void*)malloc( Size );
       if ( !*P )
         return TT_Err_Out_Of_Memory;
 
@@ -128,6 +136,89 @@
   }
 
 
+#ifdef TT_CONFIG_OPTION_EXTEND_ENGINE
+
+
+/*******************************************************************
+ *
+ *  Function    :  TT_Realloc
+ *
+ *  Description :  Reallocates memory from the heap buffer.
+ *
+ *  Input  :  Size      new size of the memory to be allocated;
+ *                      if zero or negative, TT_Free() will be called
+ *            P         pointer to a buffer pointer; if *P == NULL,
+ *                      TT_Alloc() will be called
+ *
+ *  Output :  Error code.
+ *
+ *  NOTES :  It's not necessary to zero the memory in case the 
+ *           reallocated buffer is larger than before -- the
+ *           application has to take care of this.
+ *
+ *           If the memory request fails, TT_Free() will be
+ *           called on *P, and TT_Err_Out_Of_Memory returned.
+ *
+ ******************************************************************/
+
+  EXPORT_FUNC
+  TT_Error  TT_Realloc( Long  Size, void**  P )
+  {
+    void*  Q;
+
+#ifdef DEBUG_MEMORY
+    Int  i;
+#endif
+
+
+    if ( !P )
+      return TT_Err_Invalid_Argument;
+
+    if ( !*P )
+      return TT_Alloc( Size, P );
+
+    if ( Size <= 0 )
+      return TT_Free( P );
+
+    Q = (void*)realloc( *P, Size );
+    if ( !Q )
+    {
+      TT_Free( *P );
+      return TT_Err_Out_Of_Memory;
+    }
+
+#ifdef DEBUG_MEMORY
+
+    num_realloc++;
+
+    i = 0;
+    while ( i < MAX_TRACKED_BLOCKS && pointers[i].base != *P )
+      i++;
+
+    if ( i >= MAX_TRACKED_BLOCKS )
+      fail_realloc++;
+    else
+    {
+#ifndef TT_CONFIG_REENTRANT
+      TTMemory_Allocated += Size - pointers[i].size;
+      if ( Size > pointers[i].size )
+        TTMemory_MaxAllocated += Size - pointers[i].size;
+#endif
+
+      pointers[i].base = Q;
+      pointers[i].size = size;
+    }
+#endif /* DEBUG_MEMORY */
+
+    *P = Q;
+
+    return TT_Err_Ok;
+  }
+
+
+#endif /* TT_CONFIG_OPTION_EXTEND_ENGINE */
+
+
 /*******************************************************************
  *
  *  Function    :  TT_Free
@@ -205,11 +296,13 @@
       pointers[i].size = 0;
     }
 
-    num_alloc = 0;
-    num_free  = 0;
+    num_alloc   = 0;
+    num_realloc = 0;
+    num_free    = 0;
 
-    fail_alloc = 0;
-    fail_free  = 0;
+    fail_alloc   = 0;
+    fail_realloc = 0;
+    fail_free    = 0;
 #endif
 
 
@@ -256,6 +349,11 @@
              num_alloc,
              fail_alloc );
 
+    fprintf( stderr,
+             "%d memory reallocations, of which %d failed\n",
+             num_realloc,
+             fail_realloc );
+
     fprintf( stderr,
              "%d memory frees, of which %d failed\n",
              num_free,
