@@ -9,7 +9,7 @@
  *
  *  This file is part of the FreeType project, and may only be used
  *  modified and distributed under the terms of the FreeType project
- *  license, LICENSE.TXT.  By continuing to use, modify, or distribute 
+ *  license, LICENSE.TXT.  By continuing to use, modify, or distribute
  *  this file you indicate that you have read the license and
  *  understand and accept it fully.
  *
@@ -29,8 +29,9 @@
 #ifndef TTMEMORY_H
 #define TTMEMORY_H
 
+#include "ttconfig.h"
 #include "tttypes.h"
-#include "string.h"
+#include <string.h>
 
 
 #ifdef __cplusplus
@@ -45,27 +46,62 @@
 #define MEM_Copy( dest, source, count )  bcopy( source, dest, count )
 #endif
 
+#ifdef HAVE_MEMMOVE
+#define MEM_Move( dest, source, count )  memmove( dest, source, count )
+#else
+#define MEM_Move( dest, source, count )  bcopy( source, dest, count )
+#endif
+
+
 #define MEM_Alloc( _pointer_, _size_ ) \
-          TT_Alloc( _size_, (void**)&(_pointer_) )
+  TT_Alloc( _size_, (void**)&(_pointer_) )
+
+#define MEM_Realloc( _pointer_, _size_ ) \
+  TT_Realloc( _size_, (void**)&(_pointer_) )
+
 #define ALLOC( _pointer_, _size_ ) \
-          ( error = MEM_Alloc( _pointer_, _size_ ) )
+  ( ( error = MEM_Alloc( _pointer_, _size_ ) ) != TT_Err_Ok )
+
 #define ALLOC_ARRAY( _pointer_, _count_, _type_ ) \
-          ( error = MEM_Alloc( _pointer_, (_count_)*sizeof(_type_) ) )
-#define FREE( _pointer_ )  TT_Free( (void**)&(_pointer_) )
+  ( ( error = MEM_Alloc( _pointer_, \
+                         (_count_) * sizeof ( _type_ ) ) ) != TT_Err_Ok )
+
+#define REALLOC( _pointer_, _size_ ) \
+  ( ( error = MEM_Realloc( _pointer_, _size_ ) ) != TT_Err_Ok )
+
+#define REALLOC_ARRAY( _pointer_, _count_, _type_ ) \
+  ( (error = MEM_Realloc( _pointer_, \
+                          (_count_) * sizeof ( _type_ ) ) ) != TT_Err_Ok )
+
+#define FREE( _pointer_ ) \
+  TT_Free( (void**)&(_pointer_) )
+
 
-  
   /* Allocate a block of memory of 'Size' bytes from the heap, and */
   /* sets the pointer '*P' to its address.  If 'Size' is 0, or in  */
   /* case of error, the pointer is always set to NULL.             */
 
-  TT_Error  TT_Alloc( long  Size, void**  P );
+  EXPORT_DEF
+  TT_Error  TT_Alloc( Long  Size, void**  P );
 
+#ifdef TT_CONFIG_OPTION_EXTEND_ENGINE
+
+  /* Reallocates a block of memory pointed to by '*P' to 'Size'    */
+  /* bytes from the heap, possibly changing '*P'.  If 'Size' is 0, */
+  /* TT_Free() is called, if '*P' is NULL, TT_Alloc() is called.   */
+  /* '*P' is freed (if it's non-NULL) in case of error.            */
+
+  EXPORT_DEF
+  TT_Error  TT_Realloc( Long  Size, void**  P );
+
+#endif /* TT_CONFIG_OPTION_EXTEND_ENGINE */
 
   /* Releases a block that was previously allocated through Alloc. */
   /* Note that the function returns successfully when P or *P are  */
   /* already NULL.  The pointer '*P' is set to NULL on exit in     */
   /* case of success.                                              */
 
+  EXPORT_DEF
   TT_Error  TT_Free( void**  P );
 
 
@@ -73,10 +109,10 @@
   /* Note that this won't release the previously allocated font pool. */
 
 #define Init_FontPool( x, y )  while( 0 ) { }
-  
 
-  TT_Error  TTMemory_Init();
-  TT_Error  TTMemory_Done();
+
+  LOCAL_DEF TT_Error  TTMemory_Init( void );
+  LOCAL_DEF TT_Error  TTMemory_Done( void );
 
 
 #ifdef __cplusplus
