@@ -9,7 +9,7 @@
  *
  *  This file is part of the FreeType project, and may only be used
  *  modified and distributed under the terms of the FreeType project
- *  license, LICENSE.TXT. By continuing to use, modify, or distribute
+ *  license, LICENSE.TXT.  By continuing to use, modify, or distribute
  *  this file you indicate that you have read the license and
  *  understand and accept it fully.
  *
@@ -18,9 +18,8 @@
 #ifndef TTOBJS_H
 #define TTOBJS_H
 
-#include "ttcommon.h"
+#include "ttconfig.h"
 #include "ttengine.h"
-#include "tterror.h"
 #include "ttmutex.h"
 #include "ttcache.h"
 #include "tttables.h"
@@ -139,24 +138,22 @@
 /*                                                                       */
 /*     -- Finalize the engine.  This will also destroy all pending       */
 /*        glyph objects (here 'glyph').                                  */
-/*                                                                       */
-/*                                                                       */
 
-  struct _TFace;
-  struct _TInstance;
-  struct _TExecution_Context;
-  struct _TGlyph;
+  struct TFace_;
+  struct TInstance_;
+  struct TExecution_Context_;
+  struct TGlyph_;
 
-  typedef struct _TFace  TFace;
+  typedef struct TFace_  TFace;
   typedef TFace*         PFace;
 
-  typedef struct _TInstance  TInstance;
+  typedef struct TInstance_  TInstance;
   typedef TInstance*         PInstance;
 
-  typedef struct _TExecution_Context  TExecution_Context;
+  typedef struct TExecution_Context_  TExecution_Context;
   typedef TExecution_Context*         PExecution_Context;
 
-  typedef struct _TGlyph  TGlyph;
+  typedef struct TGlyph_  TGlyph;
   typedef TGlyph*         PGlyph;
 
 
@@ -177,11 +174,11 @@
   /* the font file.  Thus, we can use 'int's   */
   /* where needed.                             */
 
-  struct  _TGraphicsState
+  struct  TGraphicsState_
   {
-    Int            rp0;
-    Int            rp1;
-    Int            rp2;
+    UShort         rp0;
+    UShort         rp1;
+    UShort         rp2;
 
     TT_UnitVector  dualVector;
     TT_UnitVector  projVector;
@@ -195,23 +192,23 @@
     TT_F26Dot6     control_value_cutin;
     TT_F26Dot6     single_width_cutin;
     TT_F26Dot6     single_width_value;
-    Int            delta_base;
-    Int            delta_shift;
+    Short          delta_base;
+    Short          delta_shift;
 
     Byte           instruct_control;
     Bool           scan_control;
     Int            scan_type;
 
-    Int            gep0;
-    Int            gep1;
-    Int            gep2;
-
+    UShort         gep0;
+    UShort         gep1;
+    UShort         gep2;
   };
 
-  typedef struct _TGraphicsState  TGraphicsState;
+  typedef struct TGraphicsState_  TGraphicsState;
 
 
-  extern const TGraphicsState  Default_GraphicsState;
+  LOCAL_DEF
+  const TGraphicsState  Default_GraphicsState;
 
 
   /*************************************************************/
@@ -222,25 +219,25 @@
   /*                                                           */
   /*************************************************************/
 
-  #define MAX_CODE_RANGES   3
+#define MAX_CODE_RANGES   3
 
-  /* There can only be 3 active code ranges at once:   */
-  /*   - the Font Program                              */
-  /*   - the CVT Program                               */
-  /*   - a glyph's instructions set                    */
+/* There can only be 3 active code ranges at once:   */
+/*   - the Font Program                              */
+/*   - the CVT Program                               */
+/*   - a glyph's instructions set                    */
 
-  #define TT_CodeRange_Font  1
-  #define TT_CodeRange_Cvt   2
-  #define TT_CodeRange_Glyph 3
+#define TT_CodeRange_Font  1
+#define TT_CodeRange_Cvt   2
+#define TT_CodeRange_Glyph 3
 
 
-  struct  _TCodeRange
+  struct  TCodeRange_
   {
     PByte  Base;
-    Int    Size;
+    Long   Size;
   };
 
-  typedef struct _TCodeRange  TCodeRange;
+  typedef struct TCodeRange_  TCodeRange;
   typedef TCodeRange*         PCodeRange;
 
 
@@ -256,132 +253,134 @@
 
   /* defines a function/instruction definition record */
 
-  struct  _TDefRecord
+  struct  TDefRecord_
   {
     Int   Range;      /* in which code range is it located ? */
-    Int   Start;      /* where does it start ?               */
+    Long  Start;      /* where does it start ?               */
     Byte  Opc;        /* function #, or instruction code     */
     Bool  Active;     /* is it active ?                      */
   };
 
-  typedef struct _TDefRecord  TDefRecord;
+  typedef struct TDefRecord_  TDefRecord;
   typedef TDefRecord*         PDefRecord;
   typedef TDefRecord*         PDefArray;
 
   /* defines a call record, used to manage function calls. */
 
-  struct  _TCallRecord
+  struct  TCallRecord_
   {
-    Int  Caller_Range;
-    Int  Caller_IP;
-    Int  Cur_Count;
-    Int  Cur_Restart;
+    Int   Caller_Range;
+    Long  Caller_IP;
+    Long  Cur_Count;
+    Long  Cur_Restart;
   };
 
-  typedef struct _TCallRecord  TCallRecord;
+  typedef struct TCallRecord_  TCallRecord;
   typedef TCallRecord*         PCallRecord;
   typedef TCallRecord*         PCallStack;  /* defines a simple call stack */
 
 
   /* This type defining a set of glyph points will be used to represent */
   /* each zone (regular and twilight) during instructions decoding.     */
-  struct  _TGlyph_Zone
+  struct  TGlyph_Zone_
   {
-    int           n_points;   /* number of points in zone */
-    int           n_contours; /* number of contours       */
+    UShort        n_points;   /* number of points in zone */
+    Short         n_contours; /* number of contours       */
 
-    PCoordinates  org_x;      /* original points coordinates */
-    PCoordinates  org_y;      /* original points coordinates */
-    PCoordinates  cur_x;      /* current points coordinates  */
-    PCoordinates  cur_y;      /* current points coordinates  */
+    TT_Vector*    org;        /* original points coordinates */
+    TT_Vector*    cur;        /* current points coordinates  */
 
     Byte*         touch;      /* current touch flags         */
-    Short*        contours;   /* contour end points          */
+    UShort*       contours;   /* contour end points          */
   };
 
-  typedef struct _TGlyph_Zone  TGlyph_Zone;
-  typedef TGlyph_Zone         *PGlyph_Zone;
+  typedef struct TGlyph_Zone_  TGlyph_Zone;
+  typedef TGlyph_Zone*         PGlyph_Zone;
+
 
 
-  
 #ifndef TT_STATIC_INTEPRETER  /* indirect implementation */
 
 #define EXEC_OPS   PExecution_Context exc,
 #define EXEC_OP    PExecution_Context exc
 #define EXEC_ARGS  exc,
 #define EXEC_ARG   exc
-  
+
 #else                          /* static implementation */
 
 #define EXEC_OPS   /* void */
 #define EXEC_OP    /* void */
 #define EXEC_ARGS  /* void */
 #define EXEC_ARG   /* void */
-  
+
 #endif
 
   /* Rounding function, as used by the interpreter */
   typedef TT_F26Dot6  (*TRound_Function)( EXEC_OPS TT_F26Dot6 distance,
                                                    TT_F26Dot6 compensation );
- 
+
   /* Point displacement along the freedom vector routine, as */
   /* used by the interpreter                                 */
-  typedef void  (*TMove_Function)( EXEC_OPS PGlyph_Zone zone,
-                                            Int         point,
-                                            TT_F26Dot6  distance );
- 
+  typedef void  (*TMove_Function)( EXEC_OPS PGlyph_Zone  zone,
+                                            UShort       point,
+                                            TT_F26Dot6   distance );
+
   /* Distance projection along one of the proj. vectors, as used */
   /* by the interpreter                                          */
-  typedef TT_F26Dot6  (*TProject_Function)( EXEC_OPS TT_F26Dot6 Vx,
-                                                     TT_F26Dot6 Vy );
-  
+  typedef TT_F26Dot6  (*TProject_Function)( EXEC_OPS TT_Vector*  v1,
+                                                     TT_Vector*  v2 );
+
   /* reading a cvt value. Take care of non-square pixels when needed */
-  typedef TT_F26Dot6  (*TGet_CVT_Function)( EXEC_OPS  Int index );
+  typedef TT_F26Dot6  (*TGet_CVT_Function)( EXEC_OPS ULong  index );
 
   /* setting or moving a cvt value.  Take care of non-square pixels  */
   /* when needed                                                     */
-  typedef void  (*TSet_CVT_Function)( EXEC_OPS  Int         index,
+  typedef void  (*TSet_CVT_Function)( EXEC_OPS  ULong       index,
                                                 TT_F26Dot6  value );
 
   /* subglyph transformation record */
-  struct  _TTransform
+  struct  TTransform_
   {
     TT_Fixed    xx, xy; /* transformation */
     TT_Fixed    yx, yy; /*     matrix     */
     TT_F26Dot6  ox, oy; /*    offsets     */
   };
 
-  typedef struct _TTransform  TTransform;
-  typedef TTransform         *PTransform;
+  typedef struct TTransform_  TTransform;
+  typedef TTransform*         PTransform;
 
   /* subglyph loading record.  Used to load composite components */
-  struct  _TSubglyph_Record
+  struct  TSubglyph_Record_
   {
-    Int          index;        /* subglyph index           */
+    Long         index;        /* subglyph index; initialized with -1 */
     Bool         is_scaled;    /* is the subglyph scaled?  */
     Bool         is_hinted;    /* should it be hinted?     */
     Bool         preserve_pps; /* preserve phantom points? */
 
     Long         file_offset;
 
+#if 0
     TT_BBox      bbox;
+    Int          leftBearing;     /* in FUnits */
+    Int          advanceWidth;    /* in FUnits */
+#endif
+
+    TT_Big_Glyph_Metrics  metrics;
 
     TGlyph_Zone  zone;
 
-    Int          arg1;  /* first argument  */
-    Int          arg2;  /* second argument */
+    Long         arg1;  /* first argument  */
+    Long         arg2;  /* second argument */
 
-    Int          element_flag;    /* current load element flag */
+    UShort       element_flag;    /* current load element flag */
 
     TTransform   transform;       /* transform */
 
     TT_Vector    pp1, pp2;        /* phantom points */
 
-    Int          leftBearing;     /* in FUnits */
-    Int          advanceWidth;    /* in FUnits */
   };
 
-  typedef struct _TSubglyph_Record  TSubglyph_Record;
+  typedef struct TSubglyph_Record_  TSubglyph_Record;
   typedef TSubglyph_Record*         PSubglyph_Record;
   typedef TSubglyph_Record*         PSubglyph_Stack;
 
@@ -440,15 +439,15 @@
   /*                                                                     */
 
   /* metrics used by the instance and execution context objects */
-  struct  _TIns_Metrics
+  struct  TIns_Metrics_
   {
     TT_F26Dot6  pointSize;      /* point size.  1 point = 1/72 inch. */
 
-    Int         x_resolution;   /* device horizontal resolution in dpi. */
-    Int         y_resolution;   /* device vertical resolution in dpi.   */
+    UShort      x_resolution;   /* device horizontal resolution in dpi. */
+    UShort      y_resolution;   /* device vertical resolution in dpi.   */
 
-    Int         x_ppem;         /* horizontal pixels per EM */
-    Int         y_ppem;         /* vertical pixels per EM   */
+    UShort      x_ppem;         /* horizontal pixels per EM */
+    UShort      y_ppem;         /* vertical pixels per EM   */
 
     Long        x_scale1;
     Long        x_scale2;    /* used to scale FUnits to fractional pixels */
@@ -460,7 +459,7 @@
     Long        x_ratio;
     Long        y_ratio;
 
-    Int         ppem;        /* maximum ppem size */
+    UShort      ppem;        /* maximum ppem size */
     Long        ratio;       /* current ratio     */
     Long        scale1;
     Long        scale2;      /* scale for ppem */
@@ -471,8 +470,8 @@
     Bool        stretched;      /* `is the glyph stretched?'-flag */
   };
 
-  typedef struct _TIns_Metrics  TIns_Metrics;
-  typedef TIns_Metrics         *PIns_Metrics;
+  typedef struct TIns_Metrics_  TIns_Metrics;
+  typedef TIns_Metrics*         PIns_Metrics;
 
 
 
@@ -482,7 +481,7 @@
   /*                                                                     */
   /***********************************************************************/
 
-  struct  _TFace
+  struct  TFace_
   {
     /* parent engine instance for the face object */
     PEngine_Instance  engine;
@@ -510,11 +509,13 @@
     /*  max values that are used to load and allocate */
     /*  further tables.                               */
 
-
     TT_Header             fontHeader;           /* the font header, as   */
                                                 /* found in the TTF file */
     TT_Horizontal_Header  horizontalHeader;     /* the horizontal header */
 
+    Bool                  verticalInfo;         /* True when vertical table */
+    TT_Vertical_Header    verticalHeader;       /* is present in the font   */
+
     TT_OS2                os2;                  /* 'OS/2' table */
 
     TT_Postscript         postscript;           /* 'Post' table */
@@ -526,49 +527,45 @@
     TGasp                 gasp;                 /* the 'gasp' table */
 
     /* The directory of TrueType tables for this typeface */
-    Int             numTables;
+    UShort          numTables;
     PTableDirEntry  dirTables;
 
     /* The directory of character mappings table for */
     /* this typeface                                 */
-    Int         numCMaps;
+    UShort      numCMaps;
     PCMapTable  cMaps;
 
     /* The glyph locations table */
-    Int       numLocations;
+    UShort    numLocations;
     PStorage  glyphLocations;
 
-    /* The HMTX table data, used to compute both left */
-    /* side bearing and advance width for all glyphs  */
-    PTableHorMetrics  longHMetrics;
-    PShort            shortMetrics;
+    /* NOTE : Tjhe "hmtx" is now part of the horizontal header */
 
     /* the font program, if any */
-    Int    fontPgmSize;
-    PByte  fontProgram;
+    ULong   fontPgmSize;
+    PByte   fontProgram;
 
     /* the cvt program, if any */
-    Int    cvtPgmSize;
-    PByte  cvtProgram;
+    ULong   cvtPgmSize;
+    PByte   cvtProgram;
 
     /* the original, unscaled, control value table */
-    Int    cvtSize;
-    PShort cvt;
+    ULong   cvtSize;
+    PShort  cvt;
 
     /* The following values _must_ be set by the */
     /* maximum profile loader                    */
 
-    Int  numGlyphs;      /* the face's total number of glyphs */
-    Int  maxPoints;      /* max glyph points number, simple and composite */
-    Int  maxContours;    /* max glyph contours number, simple and composite */
-    Int  maxComponents;  /* max components in a composite glyph */
+    UShort  numGlyphs;     /* the face's total number of glyphs */
+    UShort  maxPoints;     /* max glyph points number, simple and composite */
+    UShort  maxContours;   /* max glyph contours numb, simple and composite */
+    UShort  maxComponents; /* max components in a composite glyph */
 
     /* the following are object caches to track active */
     /* and recycled instances and execution contexts   */
     /* objects.  See 'ttcache.h'                       */
 
     TCache  instances;   /* current instances for this face */
-    TCache  contexts;    /* current execution contexts for this face */
     TCache  glyphs;      /* current glyph containers for this face */
 
 
@@ -593,7 +590,7 @@
   /*                                                                     */
   /***********************************************************************/
 
-  struct  _TInstance
+  struct  TInstance_
   {
     PFace            owner;     /* face object */
 
@@ -601,10 +598,10 @@
 
     TIns_Metrics     metrics;
 
-    Int              numFDefs;  /* number of function definitions */
+    ULong            numFDefs;  /* number of function definitions */
     PDefArray        FDefs;     /* table of FDefs entries         */
 
-    Int              numIDefs;  /* number of instruction definitions */
+    ULong            numIDefs;  /* number of instruction definitions */
     PDefArray        IDefs;     /* table of IDefs entries            */
 
     TCodeRangeTable  codeRangeTable;
@@ -612,12 +609,14 @@
     TGraphicsState   GS;
     TGraphicsState   default_GS;
 
-    Int              cvtSize;   /* the scaled control value table */
+    ULong            cvtSize;   /* the scaled control value table */
     PLong            cvt;
 
-    Int              storeSize; /* The storage area is now part of the */
+    ULong            storeSize; /* The storage area is now part of the */
     PLong            storage;   /* instance                            */
 
+    TGlyph_Zone      twilight;  /* The instance's twilight zone */
+
     /* debugging variables */
 
     /* When using the debugger, we must keep the */
@@ -638,98 +637,106 @@
   /*                                                                     */
   /***********************************************************************/
 
-  struct  _TExecution_Context 
+  struct  TExecution_Context_
   {
-    PFace           owner;
+    PFace           face;
     PInstance       instance;
 
     /* instructions state */
- 
-    Int             error;     /* last execution error */
-  
-    Int             curRange;  /* current code range number   */
-    PByte           code;      /* current code range          */
-    Int             IP;        /* current instruction pointer */
-    Int             codeSize;  /* size of current range       */
-  
-    Byte            opcode;    /* current opcode              */
-    Int             length;    /* length of current opcode    */
-  
-    Bool            step_ins;  /* true if the interpreter must */
-                                /* increment IP after ins. exec */
-  
-    Int             numFDefs;  /* number of function defs */
-    PDefRecord      FDefs;     /* table of FDefs entries  */
-  
-    Int             numIDefs;  /* number of instruction defs */
-    PDefRecord      IDefs;     /* table of IDefs entries     */
 
-    PByte           glyphIns;  /* glyph instructions buffer */
-    Int             glyphSize; /* glyph instructions buffer size */
-  
-    Int             callTop,    /* top of call stack during execution */
-                    callSize;   /* size of call stack */
-    PCallStack      callStack;  /* call stack */
-  
-    TCodeRangeTable codeRangeTable;  /* table of valid coderanges */
-                                     /* useful for the debugger   */
-  
-    Int             storeSize;  /* size of current storage */
-    PStorage        storage;    /* storage area            */
-  
-    Int             stackSize;  /* size of exec. stack */
-    Int             top;        /* top of exec. stack  */
+    TT_Error        error;     /* last execution error */
+
+    Long            top;        /* top of exec. stack  */
+
+    ULong           stackSize;  /* size of exec. stack */
     PStorage        stack;      /* current exec. stack */
-  
-    Int             args,
+
+    Long            args,
                     new_top;    /* new top after exec.    */
-  
-    TT_F26Dot6      period;     /* values used for the */
-    TT_F26Dot6      phase;      /* 'SuperRounding'     */
-    TT_F26Dot6      threshold;
-
-    TIns_Metrics    metrics;       /* instance metrics */
-
-    Int             cur_ppem;       /* ppem along the current proj vector */
-    Long            scale1;         /* scaling values along the current   */
-    Long            scale2;         /* projection vector too..            */
-    Bool            cached_metrics; /* the ppem is computed lazily. used  */
-                                    /* to trigger computation when needed */
 
     TGlyph_Zone     zp0,            /* zone records */
                     zp1,
                     zp2,
                     pts,
                     twilight;
-  
+
+    TIns_Metrics    metrics;       /* instance metrics */
+
+    TGraphicsState  GS;            /* current graphics state */
+
+    Int             curRange;  /* current code range number   */
+    PByte           code;      /* current code range          */
+    Long            IP;        /* current instruction pointer */
+    Long            codeSize;  /* size of current range       */
+
+    Byte            opcode;    /* current opcode              */
+    Int             length;    /* length of current opcode    */
+
+    Bool            step_ins;  /* true if the interpreter must */
+                                /* increment IP after ins. exec */
+    ULong           cvtSize;
+    PLong           cvt;
+
+    ULong           glyphSize; /* glyph instructions buffer size */
+    PByte           glyphIns;  /* glyph instructions buffer */
+
+    ULong           numFDefs;  /* number of function defs */
+    PDefRecord      FDefs;     /* table of FDefs entries  */
+
+    ULong           numIDefs;  /* number of instruction defs */
+    PDefRecord      IDefs;     /* table of IDefs entries     */
+
+    Int             callTop,    /* top of call stack during execution */
+                    callSize;   /* size of call stack */
+    PCallStack      callStack;  /* call stack */
+
+    UShort          maxPoints;    /* capacity of this context's "pts" */
+    UShort          maxContours;  /* record, expressed in points and  */
+                                  /* contours..                       */
+
+    TCodeRangeTable codeRangeTable;  /* table of valid coderanges */
+                                     /* useful for the debugger   */
+
+    ULong           storeSize;  /* size of current storage */
+    PLong           storage;    /* storage area            */
+
+    TT_F26Dot6      period;     /* values used for the */
+    TT_F26Dot6      phase;      /* 'SuperRounding'     */
+    TT_F26Dot6      threshold;
+
+    /* this seems to be unused */
+#if 0
+    Int             cur_ppem;       /* ppem along the current proj vector */
+#endif
+    Long            scale1;         /* scaling values along the current   */
+    Long            scale2;         /* projection vector too..            */
+    Bool            cached_metrics; /* the ppem is computed lazily. used  */
+                                    /* to trigger computation when needed */
+
     Bool            instruction_trap;  /* If True, the interpreter will */
                                        /* exit after each instruction   */
- 
-    TGraphicsState  GS;            /* current graphics state */
-                                   
+
     TGraphicsState  default_GS;    /* graphics state resulting from  */
                                    /* the prep program               */
     Bool            is_composite;  /* ture if the glyph is composite */
 
-    Int             cvtSize;
-    PLong           cvt;
-  
     /* latest interpreter additions */
-  
+
     Long               F_dot_P;    /* dot product of freedom and projection */
                                    /* vectors                               */
     TRound_Function    func_round; /* current rounding function             */
-    
+
     TProject_Function  func_project,   /* current projection function */
                        func_dualproj,  /* current dual proj. function */
                        func_freeProj;  /* current freedom proj. func  */
- 
+
     TMove_Function     func_move;      /* current point move function */
 
     TGet_CVT_Function  func_read_cvt;  /* read a cvt entry              */
     TSet_CVT_Function  func_write_cvt; /* write a cvt entry (in pixels) */
     TSet_CVT_Function  func_move_cvt;  /* incr a cvt entry (in pixels)  */
 
+    ULong              loadSize;
     PSubglyph_Stack    loadStack;      /* loading subglyph stack */
   };
 
@@ -740,27 +747,26 @@
   /*                                                                     */
   /***********************************************************************/
 
-  struct _TGlyph
+  struct TGlyph_
   {
-    PFace             face;
-    TT_Glyph_Metrics  metrics;
-    TT_Outline        outline;
+    PFace                 face;
+    TT_Big_Glyph_Metrics  metrics;
+    TT_Outline            outline;
   };
 
 
   /* The following type is used to load a font from a collection. */
   /* See Face_Create in ttobjs.c                                  */
 
-  struct  _TFont_Input
+  struct  TFont_Input_
   {
     TT_Stream         stream;     /* input stream                */
-    Int               fontIndex;  /* index of font in collection */
+    ULong             fontIndex;  /* index of font in collection */
     PEngine_Instance  engine;     /* parent engine instance      */
 
   };
 
-  typedef struct _TFont_Input  TFont_Input;
-
+  typedef struct TFont_Input_  TFont_Input;
 
 
   /********************************************************************/
@@ -768,52 +774,68 @@
   /*   Code Range Functions                                           */
   /*                                                                  */
   /********************************************************************/
- 
+
   /* Goto a specified coderange */
-  TT_Error  Goto_CodeRange( PExecution_Context  exec, Int  range, Int  IP );
- 
+  LOCAL_DEF
+  TT_Error  Goto_CodeRange( PExecution_Context  exec,
+                            Int                 range,
+                            Long                IP );
+
+#if 0
   /* Return a pointer to a given coderange record. */
   /* Used only by the debugger.                    */
-  PCodeRange  Get_CodeRange( PExecution_Context  exec, Int  range );
- 
+  LOCAL_DEF
+  PCodeRange  Get_CodeRange( PExecution_Context  exec,
+                             Int                 range );
+#endif
+
   /* Set a given code range properties */
+  LOCAL_DEF
   TT_Error  Set_CodeRange( PExecution_Context  exec,
                            Int                 range,
                            void*               base,
-                           Int                 length );
- 
+                           Long                length );
+
   /* Clear a given coderange */
+  LOCAL_DEF
   TT_Error  Clear_CodeRange( PExecution_Context  exec, Int  range );
 
 
+  LOCAL_DEF
   PExecution_Context  New_Context( PFace  face );
 
+  LOCAL_DEF
   TT_Error  Done_Context( PExecution_Context  exec );
 
 
+  LOCAL_DEF
   TT_Error  Context_Load( PExecution_Context  exec,
+                          PFace               face,
                           PInstance           ins );
 
+  LOCAL_DEF
   TT_Error  Context_Save( PExecution_Context  exec,
                           PInstance           ins );
 
+  LOCAL_DEF
   TT_Error  Context_Run( PExecution_Context  exec,
                          Bool                debug );
 
+  LOCAL_DEF
   TT_Error  Instance_Init( PInstance  ins );
 
-  TT_Error  Instance_Reset( PInstance  ins,
-                            Bool       debug );
+  LOCAL_DEF
+  TT_Error  Instance_Reset( PInstance  ins );
+
 
- 
   /********************************************************************/
   /*                                                                  */
   /*   Handy scaling functions                                        */
   /*                                                                  */
   /********************************************************************/
 
-  TT_Pos   Scale_X( PIns_Metrics  metrics, TT_Pos  x );
-  TT_Pos   Scale_Y( PIns_Metrics  metrics, TT_Pos  y );
+  LOCAL_DEF TT_Pos   Scale_X( PIns_Metrics  metrics, TT_Pos  x );
+  LOCAL_DEF TT_Pos   Scale_Y( PIns_Metrics  metrics, TT_Pos  y );
 
   /********************************************************************/
   /*                                                                  */
@@ -826,8 +848,8 @@
   /*                                                                  */
   /********************************************************************/
 
-  TT_Error  TTObjs_Init( PEngine_Instance  engine );
-  TT_Error  TTObjs_Done( PEngine_Instance  engine );
+  LOCAL_DEF TT_Error  TTObjs_Init( PEngine_Instance  engine );
+  LOCAL_DEF TT_Error  TTObjs_Done( PEngine_Instance  engine );
 
 #ifdef __cplusplus
   }
