@@ -6,7 +6,6 @@
 /*  D. Turner, R.Wilhelm, and W. Lemberg                                    */
 /*                                                                          */
 /*  ftstring: Making string text from individual glyph information.         */
-/*            This program makes use of FreeType's kerning extension.       */
 /*                                                                          */
 /*  Keys:                                                                   */
 /*                                                                          */
@@ -25,30 +24,18 @@
 /*                                                                          */
 /****************************************************************************/
 
-#ifdef ARM
-#include "std.h"
-#include "graflink.h"
-#endif
-
-#include "freetype.h"
-#include "ftxkern.h"
-#include "tterror.h"  /* for Panic() and Message() only */
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
+#include "freetype.h"
+#include "common.h"  /* for Panic() and Message() only */
+
 #include "gmain.h"
 #include "gevents.h"
 #include "gdriver.h"
 #include "display.h"
 
-#ifdef ARM
-#include "armsup.c" /* pull in our routines */
-#endif
-
-#include "common.h"
-
 #define  Pi         3.1415926535
 
 #define  MAXPTSIZE  500                 /* dtp */
@@ -99,12 +86,12 @@
 
   static void  CharToUnicode( char*  source )
   {
-    int    i, n;
-    short  platform, encoding;
+    unsigned short  i, n;
+    unsigned short  platform, encoding;
 
     /* First, look for a Unicode charmap */
 
-    n = TT_Get_CharMap_Count( face );
+    n = properties.num_CharMaps;
 
     for ( i = 0; i < n; i++ )
     {
@@ -142,7 +129,7 @@
     if ( (error = TT_Set_Instance_CharSize( instance, pointSize*64 )) )
     {
       RestoreScreen();
-      Message( "error = %d\n", error );
+      printf( "error = 0x%x\n", (int)error );
       Panic( "could not reset instance\n" );
     }
 
@@ -259,12 +246,12 @@
     {
     case event_Quit:            /* ESC or q */
       return 0;
-      
+
     case event_Keyboard:
       if ( event->info == 'h' ) /* Toggle hinting */
         hinted = !hinted;
       break;
-      
+
     case event_Rotate_Glyph:
       break;
 
@@ -284,21 +271,22 @@
 
   static void  usage( char*  execname )
   {
-    Message( "\n" );
-    Message( "ftstring: simple String Test Display -- part of the FreeType project\n" );
-    Message( "--------------------------------------------------------------------\n" );
-    Message( "\n" );
-    Message( "Usage: %s [options below] ppem fontname[.ttf|.ttc] [string]\n",
+    printf( "\n" );
+    printf( "ftstring: simple String Test Display -- part of the FreeType project\n" );
+    printf( "--------------------------------------------------------------------\n" );
+    printf( "\n" );
+    printf( "Usage: %s [options below] ppem fontname[.ttf|.ttc] [string]\n",
              execname );
-    Message( "\n" );
-    Message( "  -g     gray-level rendering (default: none)\n" );
-    Message( "\n" );
+    printf( "\n" );
+    printf( "  -g     gray-level rendering (default: none)\n" );
+    printf( "  -r R   use resolution R dpi (default: 96 dpi)\n" );
+    printf( "\n" );
 
-    exit( 1 );
+    exit( EXIT_FAILURE );
   }
 
 
-  void  main( int  argc, char**  argv ) 
+  int  main( int  argc, char**  argv )
   {
     int    i, old_ptsize, orig_ptsize, file;
     int    XisSetup = 0;
@@ -306,16 +294,17 @@
     char   alt_filename[128 + 4];
     char*  execname;
     int    option;
+    int    res = 96;
 
     TT_Error  error;
     TEvent    event;
 
- 
+
     execname = argv[0];
 
     while ( 1 )
     {
-      option = getopt( argc, argv, "g" );
+      option = ft_getopt( argc, argv, "gr:" );
 
       if ( option == -1 )
         break;
@@ -326,14 +315,20 @@
         gray_render = 1;
         break;
 
+      case 'r':
+        res = atoi( ft_optarg );
+        if ( res < 1 )
+          usage( execname );
+        break;
+
       default:
         usage( execname );
         break;
       }
     }
 
-    argc -= optind;
-    argv += optind;
+    argc -= ft_optind;
+    argv += ft_optind;
 
     if ( argc <= 1 )
       usage( execname );
@@ -346,7 +341,7 @@
     /* Initialize engine */
 
     if ( (error = TT_Init_FreeType( &engine )) )
-      Panic( "Error while initializing engine, code = %d.\n", error );
+      Panic( "Error while initializing engine, code = 0x%x.\n", error );
 
     ptsize = orig_ptsize;
     hinted = 1;
@@ -356,7 +351,7 @@
     {
       if ( argv[file][i] == '.' )
         i = 0;
-        i--; 
+        i--;
     }
 
     filename[128] = '\0';
@@ -384,14 +379,15 @@
     if ( error == TT_Err_Could_Not_Open_File )
       Panic( "Could not find/open %s.\n", filename );
     else if (error)
-      Panic( "Error while opening %s, error code = %x.\n", filename, error );
+      Panic( "Error while opening %s, error code = 0x%x.\n",
+             filename, error );
 
     /* get face properties and allocate preload arrays */
 
     TT_Get_Face_Properties( face, &properties );
 
     num_glyphs = properties.num_Glyphs;
-        
+
     /* create glyph */
 
     error = TT_New_Glyph( face, &glyph );
@@ -404,7 +400,7 @@
     if ( error )
       Panic( "Could not create instance for %s.\n", filename );
 
-    error = TT_Set_Instance_Resolutions( instance, 96, 96 );
+    error = TT_Set_Instance_Resolutions( instance, res, res );
     if ( error )
       Panic( "Could not set device resolutions." );
 
@@ -412,7 +408,7 @@
     {
       XisSetup = 1;
 
-      if ( gray_render ) 
+      if ( gray_render )
       {
         if ( !SetGraphScreen( Graphics_Mode_Gray ) )
           Panic( "Could not set up grayscale graphics mode.\n" );
@@ -425,7 +421,7 @@
           Panic( "Could not set up mono graphics mode.\n" );
       }
     }
-  
+
     Init_Display( gray_render );
 
     Reset_Scale( ptsize );
@@ -451,14 +447,15 @@
       sprintf( Header, "%s:   ptsize: %4d  hinting: %s",
                        ft_basename( filename ), ptsize,
                        hinted ? "on" : "off" );
+
+      Display_Bitmap_On_Screen( Bit.bitmap, Bit.rows, Bit.cols );
+
 #ifndef X11
 #ifndef OS2
       Print_XY( 0, 0, Header );
 #endif
 #endif
 
-      Display_Bitmap_On_Screen( Bit.bitmap, Bit.rows, Bit.cols );
-
       Get_Event( &event );
       if ( !( key = Process_Event( &event ) ) )
         goto Fin;
@@ -477,10 +474,12 @@
 
     TT_Done_FreeType( engine );
 
-    Message( "Execution completed successfully.\n" );
-    Message( "Fails = %d.\n", Fail );
+    printf( "Execution completed successfully.\n" );
+    printf( "Fails = %d.\n", Fail );
 
-    exit( 0 );
+    exit( EXIT_SUCCESS );      /* for safety reasons */
+
+    return 0;       /* never reached */
 }
 
 
