@@ -15,9 +15,8 @@
 #define PROGNAME "ftstrpnm"
 
 #include "freetype.h"
-#include "tterror.h"  /* for Panic() and Message() only */
 
-#include "common.h"   /* for getopt() */
+#include "common.h"   /* for ft_getopt() */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -32,6 +31,8 @@
   TT_Face      face;
   TT_Instance  instance;
 
+  TT_Face_Properties  properties;
+
   TT_Raster_Map  bit;
   TT_Raster_Map  small_bit;   /* used when font-smoothing is enabled */
 
@@ -73,7 +74,7 @@
     }
 
     bit->bitmap = (void *) malloc( bit->size );
-    if ( !bit->bitmap ) 
+    if ( !bit->bitmap )
       Panic( "Not enough memory to allocate bitmap!\n" );
   }
 
@@ -132,7 +133,7 @@
   {
     /* kudos for this code snippet go to Norman Walsh */
 
-    char*  bmap; 
+    char*  bmap;
     int    i;
 
 
@@ -161,16 +162,16 @@
 
   static void  Load_Glyphs( char*  txt, int  txtlen )
   {
-    int         i, n, code, load_flags;
-    int         num_glyphs = 0, no_cmap = 0;
-    short       platform, encoding;
-    TT_Error    error;
-    TT_CharMap  char_map;
+    unsigned short  i, n, code, load_flags;
+    unsigned short  num_glyphs = 0, no_cmap = 0;
+    unsigned short  platform, encoding;
+    TT_Error        error;
+    TT_CharMap      char_map;
 
 
     /* First, look for a Unicode charmap */
 
-    n = TT_Get_CharMap_Count( face );
+    n = properties.num_CharMaps;
 
     for ( i = 0; i < n; i++ )
     {
@@ -216,16 +217,12 @@
 
       if ( no_cmap )
       {
-        code = j - ' ' + 1;
-        if ( code < 0 || code >= num_glyphs )
+        code = (j - ' ' + 1) < 0 ? 0 : (j - ' ' + 1);
+        if ( code >= num_glyphs )
           code = 0;
       }
       else
-      {
         code = TT_Char_Index( char_map, j );
-        if ( code < 0 )
-          code = 0;  /* FIXME! default code */
-      }
 
       (void)(
         ( error = TT_New_Glyph( face, &glyphs[j] ) ) ||
@@ -233,7 +230,7 @@
       );
 
       if ( error )
-        Panic( "Cannot allocate and load glyph: %d.\n", error );
+        Panic( "Cannot allocate and load glyph: error 0x%x.\n", error );
     }
   }
 
@@ -270,20 +267,23 @@
       if ( error == TT_Err_Could_Not_Open_File )
         Panic( "Could not find/open %s.\n", filename );
       else
-        Panic( "Error while opening %s, error code = %x.\n",
+        Panic( "Error while opening %s, error code = 0x%x.\n",
                filename, error );
     }
- 
+
+    TT_Get_Face_Properties( face, &properties );
+
     /* create and initialize instance */
- 
+
     (void) (
       ( error = TT_New_Instance( face, &instance ) ) ||
-      ( error = TT_Set_Instance_Resolutions( instance, 96, 96 ) ) ||
+      ( error = TT_Set_Instance_Resolutions( instance, dpi, dpi ) ) ||
       ( error = TT_Set_Instance_CharSize( instance, ptsize*64 ) )
     );
 
     if ( error )
-      Panic( "Could not create and initialize instance: %d.\n", error );
+      Panic( "Could not create and initialize instance: error 0x%x.\n",
+             error );
   }
 
 
@@ -328,8 +328,6 @@
       pnm_width += gmetrics.advance / 64;
     }
 
-    /* ??? remove rsb of the last glyph here */
-
     Init_Raster_Map( &bit, pnm_width, pnm_height );
     Clear_Raster_Map( &bit );
 
@@ -408,23 +406,23 @@
 
   static void  usage( void )
   {
-    Message( "\n" );
-    Message( "%s: simple text to image converter -- part of the FreeType project\n", PROGNAME );
-    Message( "\n" );
-    Message( "Usage: %s [options below] filename [string]\n", PROGNAME );
-    Message( "\n" );
-    Message( "  -g     gray-level rendering (default: off)\n" );
-    Message( "  -h     hinting off (default: on)\n" );
-    Message( "  -r X   resolution X dpi (default: 96)\n" );
-    Message( "  -p X   pointsize X pt (default: 12)\n" );
-    Message( "  -b X   border X pixels wide (default: 0)\n" );
-    Message( "\n" );
+    printf( "\n" );
+    printf( "%s: simple text to image converter -- part of the FreeType project\n", PROGNAME );
+    printf( "\n" );
+    printf( "Usage: %s [options below] filename [string]\n", PROGNAME );
+    printf( "\n" );
+    printf( "  -g     gray-level rendering (default: off)\n" );
+    printf( "  -h     hinting off (default: on)\n" );
+    printf( "  -r X   resolution X dpi (default: 96)\n" );
+    printf( "  -p X   pointsize X pt (default: 12)\n" );
+    printf( "  -b X   border X pixels wide (default: 0)\n" );
+    printf( "\n" );
 
-    exit( 1 );
+    exit( EXIT_FAILURE );
   }
 
 
-  void  main( int  argc, char**  argv ) 
+  int  main( int  argc, char**  argv )
   {
     int       option, txtlen;
     char      *txt, *filename;
@@ -435,7 +433,7 @@
 
     while ( 1 )
     {
-      option = getopt( argc, argv, "ghr:p:b:" );
+      option = ft_getopt( argc, argv, "ghr:p:b:" );
 
       if ( option == -1 )
         break;
@@ -449,13 +447,13 @@
         hinted = 0;
         break;
       case 'r':
-        dpi = atoi( optarg );
+        dpi = atoi( ft_optarg );
         break;
       case 'p':
-        ptsize = atoi( optarg );
+        ptsize = atoi( ft_optarg );
         break;
       case 'b':
-        border = atoi( optarg );
+        border = atoi( ft_optarg );
         break;
 
       default:
@@ -464,10 +462,10 @@
       }
     }
 
-    argc -= optind;
-    argv += optind;
+    argc -= ft_optind;
+    argv += ft_optind;
 
-    if ( argc <= 0 || argc > 2 || dpi <= 0 || ptsize <= 0 )
+    if ( argc <= 0 || argc > 2 || dpi <= 0 || ptsize <= 0 || border < 0 )
       usage();
 
     filename = argv[0];
@@ -483,7 +481,7 @@
 
     error = TT_Init_FreeType( &engine );
     if ( error )
-      Panic( "Error while initializing engine, code = %d.\n", error );
+      Panic( "Error while initializing engine, code = 0x%x.\n", error );
 
     Init_Face( filename );
     Load_Glyphs( txt, txtlen );
@@ -504,7 +502,9 @@
 
     TT_Done_FreeType( engine );
 
-    exit( 0 );
+    exit( EXIT_SUCCESS );      /* for safety reasons */
+
+    return 0;       /* never reached */
 }
 
 
